; Heart Rate Monitor Interface (HRMI) alternative firmware with averaging code stripped out
; and returning the period between pulses from the RMCM01 receiver..
; File: hrmi_period.asm
;
; This software is provided "as-is" and copyright holders make no representations or
; warranties, express or implied, including but not limited to, warranties of merchantability
; or fitness for any particular purpose or that the use of the software will not infringe
; any third party patents, copyrights, trademarks or other rights.
;
; Copyright holders will not be liable for any direct, indirect, special or consequential
; damages arising out of use of the software.
;
; Title to copyright in this software will at all times remain with copyright holders.
;
; Copyright (c) 2007-2011 Dan Julio  All rights reserved
; 
;
; Global Assembler directives
	list		c=132, n=80
	list		p=16F913
	errorlevel	-302	; Disable messages for register banking
	errorlevel	-306	; Disable messages for page crossing
;
; This program provides raw heart rate data from a Polar RMCM01 OEM
; coded/non-coded receiver via a serial or I2C interface.  The program
; measures and stores the time between pulses from the RMCM01.
; It also provides access to analog inputs via the microcontroller's
; ADC port and several I/O pins that can be configured by the user.
;
; RMCM01 Interface
; ----------------
; The firmware allows access to the raw timing data from the RMCM01.  It returns
; the period (in milliseconds) between consecutive pulses from the RMCM01 that
; fall within legal values for a heartbeat period (approx 33 - 240 bpm).
; The valid period information is stored in a history buffer of the last 16
; detected heart beats.
;
; The firmware asserts the HRreset signal for 10 mSec after a rewaiting period
; has expired with no heart-rate signal to allow the RMCM01 to more quickly detect
; a different coded transmitter.  To allow the microcontroller to operate at a higher
; voltage than the RMCM01, the reset signal is only enabled when it will be
; driven low.  An external pull-up referenced to the RMCM01 power supply rail
; keeps the signal high when it is not being asserted.
;
; This firmware will operate with either the RMCM01 HR or FPLS outputs (filters
; out coded transmitter indication within the 250 mSec window on the FPLS output).
;
; Analog Inputs
; -------------
; The firmware provides access to the microcontroller's Analog to Digital
; converter.  The ADC is configured to use the microcontroller's VCC input
; as a voltage reference.  The four Analog inputs (AN0-AN3) may be read
; in 10-bit mode.
;
; Utility Port
; ------------
; The firmware provides access to 5 microcontroller I/O pins.  The direction
; of each pin (input or output) can be controlled.  The pins may be read or
; written (outputs).  From the factory the default state for each pin is an
; output driving a low level.  The default state may be changed by the user.
;
; User Interface
; --------------
; The firmware controls a LED to communicate status to the user.  The 
; following information is communicated:
;    Solid : Searching for Heart Rate data
;    Blinking : Heart rate data (blink corresponds to measured heart rate)
;
; The firmware samples 8 inputs at power-on to configure operation.  One input
; determines the interface type, either serial data or I2C.  The other inputs
; depend on the interface type (config inputs).  For a serial interface they
; specify the baud rate.  For a I2C interface they specify the 7-bit device address. 
; The config inputs utilize the built-in microcontroller pull-ups and may be left
; floating for a "1" input or grounded for a "0" input.  The interface type
; requires an external pull-up.
;
; Interface Type = 1 (Serial interface)
;
;   Config[6:0]      Baud Rate
;   --------------------------
;     xxxxx00        2400 baud
;     xxxxx01       19200 baud
;     xxxxx10       38400 baud
;     xxxxx11        9600 baud (default with no jumpers)
;
; Interface Type = 0 (I2C interface)
;
;   Config[6:0]      Address
;   --------------------------
;     0000000           0
;     0000001           1
;      ...
;     1111111         127 (default with no jumpers)
; 
; Serial Interface
; ----------------
; The device acts as a slave when the interface is configured as a serial interface.
; Data is transmitted with 8-bits, no parity and 1 stop bit (8N1).  The firmware 
; responds to packets having the following layout:
;
;    <CommandChar> [ [<space>] <Argument>] <CR>
;
; where
;
;    <CommandChar> is a single ASCII character specifying the packet type.
;
;    <space> is the space character (0x20).  The space character is optional.
;
;    <Argument> is an 8-bit decimal numeric argument specified as 1 to 3 ASCII
;      characters '0' through '9' ("0" through "255").  Numeric values greater
;      than 255 are set to 255 internally by the firmware.  The argument field
;      is present only for commands which require it.
;
;    <CR> is the linefeed character (0x0D).  It is used to terminate a packet.
;
; All other character codes are ignored by the firmware.  Packets must be correctly
; constructed and end with a <CR> character.  Data that is not a <CR> character
; following the argument field is ignored.  Characters other than one of the valid
; packet types are ignored when starting a new packet (allowing for a <LF> to follow
; the <CR> character).
;
; The following packet types are interpreted by the firmware:
;
;     A : Read an Analog input and return a 8-bit result.
;        "A <N> <CR>" where N specifies the port 0 - 3 ("0" through "3")
;
;     D : Set the I/O directions of each bit in the utility port.
;        "D <N> <CR>" where N is a 5-bit mask specifying the direction of each
;                     utility port bit (a value of 0 in a bit position makes
;                     that bit an output, a value of 1 in a bit position makes
;                     that bit an input).  Values for N are 0 - 31 ("0" through
;                     "31").  Bits above the 5-bit mask set for values greater
;                     than 31 are ignored.
;
;     G : Get heart rate data.
;        "G <N> <CR>" where N is the number of heart rate samples from the history
;                     buffer to return.  Values for N are 0 - 16 ("0" through
;                     "16").  Values greater than 16 are converted to 16 internally
;                     by the firmware.
;
;     I : Get the current values of the utility port pins.
;        "I <CR>"
;  
;     M : Get the current operating mode.
;        "M <CR>"
;
;     O : Set the values of the utility port pins.
;        "O <N> <CR>" where N is a 5-bit mask containing values to set on each
;                     utility port pin defined as an output.  Bits in locations
;                     defined as inputs are ignored.  Values for N are 0 - 31 ("0"
;                     through "31").  Bits above the 5-bit mask set for values greater
;                     than 31 are ignored.
;
;     S : Set the current operating mode.
;        "S <N> <CR>" where N is a 1-bit mask bits specifying the operating mode
;                     of the firmware and the power-up configuration.
;
;                     Bit       Function
;                     ------------------------------------------------------------------
;                      2        Trigger bit to store the current Utility port configuration
;                               as the power-on default.  The current I/O directions are
;                               stored.  The current values of any output pins are also
;                               stored.
;
;                      Values for N are 0 or 4 ("0" or "4").  All other bits are ignored.
;
;     V : Get the firmware version.
;        "V <CR>"
;
; Certain commands will cause the firmware to generate a response packet and transmit
; that back through the serial interface.  The firmware can generate the following response
; packets (for the specified command):
;
;     A (Get analog input): "<Value> <CR>"
;      <Value> : An 10-bit value from the selected analog port.  Values range from 0 -
;                1023 ("0" through "1023").
;
;     G (Get heart rate data): "<Status> <Count> [ <HR1P> [<HR2P> [<HR3P> ... <HR16P>]]] <CR>"
;      <Status> : A 1-bit flag indicating the current status of the heart rate acquisition
;                 algorithm.
;                 7:2: Reserved: 0
;                   1: No Data (1 = No HR pulses detected, 0 = At least 2 HR pulses detected)
;                   0: Reserved: 0
;                 Values range from 0 - 14 ("0" through "14").
;      <Count> : An 8-bit value indicating the current second count in averaged mode 
;                and the heartbeat count in immediate mode (modulo 256) in the HRMI 
;                allowing external software to identify new heart rates in the data 
;                from the last time the HRMI was polled.  Values range from 0 - 255
;                ("0" through "255").
;      <HR1P> : The most recent heart rate period (recorded at time <Count>).
;      <HR2P> : The previous heart rate period (and so on for <HR3P>...).
;              A value of 0 for a heart rate period indicates that the HRMI was unable
;              to calculate a heart rate (no heart rate data or bad heart rate data)
;              for that sample period.
;              Heart rate period values may range from 250 - 1792 mSec ("250" - "1792").
;
;     I (Get Utility pins): "<Value> <CR>"
;      <Value> : The current 5-bit value of the utility pins.  Values range from 0 - 31
;                ("0" through "31").
;
;     M (Get operating mode): "<Dir> <CR>"
;      <Dir>  : The current Utility port I/O direction configuration.  Values range
;               from 0 - 31 ("0" through "31").
;
;     V (Get firmware version): "<Type> <Version> <CR>"
;      <Type> : The 8-bit firmware type.  This firmware is type 0x02 and will response
;               with the value 2 ("2").
;      <Version> : The 8-bit firmware version.  The version is incremented for each
;                  released version of firmware starting with the value 1 ("1").
;
; I2C Interface
; -------------
; The firmware implements a I2C slave with 7-bit addressing.  It supports both
; Standard Mode (100 kbits/sec)).  It does not support the optional 10-bit addressing,
; General Call addressing, Software Reset, START byte or Device ID features.
;
; The firmware responds to the same commands as in serial mode.  In I2C mode all
; data is transmitted and interpreted as binary bytes.
;
;     A (Get analog value): 0x41 <N>
;       <N> specifies the port (0 - 3).
;
;       Requires a subsequent 2-byte I2C read to obtain the 10-bit result.
;       Data is returned with the most significant 2-bits in bit positions 1:0
;       of the first byte and the least significant 8-bits in the second byte.
;       <Data[9:8]> <Data[7:0]>
;
;     D (Set Utility Port I/O directions): 0x44 <N>
;       <N> bits 4:0 specify the direction mask (0: Output, 1: Input).
;
;     G (Get heart rate data): 0x47 <N>
;       <N> specifies the number of samples to return (0 - 16).
;
;       Requires a subsequent 2*N + 4 byte I2C read to obtain the heart rate data.
;         <Status> <Count> [<HR1P> [<HR2P> ... ]]
;       <Status> is returned as <0x00><Status>
;       <Count> is returned as <0x00><Count>
;       Heart rate period data is returned high byte first: HR1P = <15:8><7:0>
;
;     I (Get Utility Port values): 0x49 
;
;       Requires a subsequent 2 byte I2C read to obtain the Utility Port value
;       in bits 4:0.
;       <0x00><Value>
;
;     M (Get current operating mode): 0x4D 
;
;       Requires a subsequent 2-byte I2C read to obtain the current 
;       current Utility Port I/O direction configuration (bits 4:0).
;       <0x00> <Dir>
;
;     O (Set Utility Port pins): 0x4F <N>
;       <N> bits 4:0 specify the value for output pins.
;
;     S (Set mode): 0x53 <N>
;       <N> bit 2 stores the utility port power-up configuration (direction, power-on state).
;
;     V (Get firmware version): 0x56 
;
;       Requires a subsequent 4-byte I2C read to obtain the firmware type
;       in the first 16-bits and the firmware version in the second 16-bits.
;       <0x00><Type> <0x00><Version>
;
; Commands requiring a response cause the firmware to load a response buffer in
; preparation for the subsequent I2C read.  The buffer is overwritten by the next
; command requiring a response.  Data from one command will be lost if another
; command is issued before the read for the first command is executed.
;
;
;
; Revision History
;   04/28/11 Revision 1.0  DJJ  First pass based on hrmi.asm revision 1.0
;     
;
;
; ------------------------------------------------------------------------
; Processor Information:
;
; Target processor: PIC 16C913
; Processor frequency: Internal 8 MHz (500 nS instruction cycle)
;
; Memory Layout
;    Page 0: 0x0000 - 0x07FF	ISR, Main code, main code subroutines
;    Page 1: 0x0800 - 0x0FFF	ISR routines
;
; Input/Output Pin Assignments (PDIP/SOIC package)
;
;   Pin     Signal                       Polarity      I/O     Default State
;   ------------------------------------------------------------------------
;    2 RA0  Analog in[0]                 -             Input   0
;    3 RA1  Analog in[1]                 -             Input   0
;    4 RA2  Analog in[2]                 -             Input   0
;    5 RA3  Analog in[3]                 -             Input   0
;    6 RA4  Serial / ~I2C interface      Active High   Input   0 (external pullup)
;    7 RA5  RMCM01 reset                 Active Low    I/O     1 (only enabled to reset)
;   10 RA6  Reserved for ext xtal        Active High   Output  0
;    9 RA7  Reserved for ext xtal        Active High   Output  0
;   21 RB0  HR input                     Active High   Input   0 (IRQ/internal pullup)
;   22 RB1  Config Input [0]             Active High   Input   0 (internal pullup)
;   23 RB2  Config Input [1]             Active High   Input   0 (internal pullup)
;   24 RB3  Config Input [2]             Active High   Input   0 (internal pullup)
;   25 RB4  Config Input [3]             Active High   Input   0 (internal pullup)
;   26 RB5  Config Input [4]             Active High   Input   0 (internal pullup)
;   27 RB6  Config Input [5] / ICSP CLK  Active High   Input   0 (internal pullup)
;   28 RB7  Config Input [6] / ICSP DAT  Active High   Input   0 (internal pullup)
;   11 RC0  Utility Port[0]              Active High   I/O     0
;   12 RC1  Utility Port[1]              Active High   I/O     0
;   13 RC2  Utility Port[2]              Active High   I/O     0
;   14 RC3  Utility Port[3]              Active High   I/O     0
;   15 RC4  Utility Port[4]              Active High   I/O     0
;   16 RC5  Status LED                   Active Low    Output  1
;   17 RC6  TX / SCL                     Active High   Output  0 (Input for I2C)
;   18 RC7  RX / SDA                     Active High   Input   0 (I/O for I2C)
;    1 nMCLR Reset input
;
; TIMER Utilization
;    TIMER0: Used to time RMCM01 reset during operation
;    TIMER1: Used to set blink (on-period) for status indicators
;    TIMER2: Used to generate an interrupt every 1 mSec
;
;
;
; ------------------------------------------------------------------------
; File Layout:
;
;    1. Configuration Bits
;    2. Program Constants
;    3. Macro definitions
;    4. Data EEPROM assembler initialization directives
;    5. Main code
;    6. Main code subroutines
;    7. Interrupt routines
;    8. Interrupt subroutines
;
;
; ------------------------------------------------------------------------

;==========================================================================
;
;       RAM Definition
;
;==========================================================================

        __MAXRAM H'1FF'
        __BADRAM H'08', H'1B'-H'1D'
        __BADRAM H'88', H'9A'-H'9B'
        __BADRAM H'112', H'115', H'118', H'11B',H'11E'-H'11F'
        __BADRAM H'185', H'187'-H'189', H'18E'-H'1EF'

;==========================================================================
;
;       Configuration Bits
;
;==========================================================================

_CONFIG                      EQU     H'2007'

;Configuration Byte 1 Options
_DEBUG_ON		     EQU     H'2FFF'
_DEBUG_OFF                   EQU     H'3FFF'
_FCMEN_ON                    EQU     H'3FFF'
_FCMEN_OFF                   EQU     H'37FF'
_IESO_ON                     EQU     H'3FFF'
_IESO_OFF                    EQU     H'3BFF'
_BOD_ON                      EQU     H'3FFF'
_BOD_NSLEEP                  EQU     H'3EFF'
_BOD_SBODEN                  EQU     H'3DFF'
_BOD_OFF                     EQU     H'3CFF'
_CPD_ON                      EQU     H'3F7F'
_CPD_OFF                     EQU     H'3FFF'
_CP_ON                       EQU     H'3FBF'
_CP_OFF                      EQU     H'3FFF'
_MCLRE_ON                    EQU     H'3FFF'
_MCLRE_OFF                   EQU     H'3FDF'
_PWRTE_ON                    EQU     H'3FEF'
_PWRTE_OFF                   EQU     H'3FFF'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FF7'
_LP_OSC                      EQU     H'3FF8'
_XT_OSC                      EQU     H'3FF9'
_HS_OSC                      EQU     H'3FFA'
_EC_OSC                      EQU     H'3FFB'
_INTRC_OSC_NOCLKOUT          EQU     H'3FFC'
_INTRC_OSC_CLKOUT            EQU     H'3FFD'
_EXTRC_OSC_NOCLKOUT          EQU     H'3FFE'
_EXTRC_OSC_CLKOUT            EQU     H'3FFF'
_INTOSCIO                    EQU     H'3FFC'
_INTOSC                      EQU     H'3FFD'
_EXTRCIO                     EQU     H'3FFE'
_EXTRC                       EQU     H'3FFF'

; Set the device configuration bits
;    DEBUG = 1 (RB6/7 are general purpose IO pins)
;    FCMEN = 0 (Fail-Safe Clock Monitor disabled)
;    IESO = 0 (Internal/External Switchover disabled)
;    BOREN1:0 = 00 (Brown-out Reset disabled)
;    CPD = 0 (Data EEPROM protection off)
;    CP = 0 (Code protection off)
;    MCLRE = 1 (RE3 is nMCLR)
;    PWRTEN = 0 (Power-up Timer enabled)
;    WDTEN = 1 (WDT enabled)
;    FOSC2:FOSC0 = 100 (INTRC oscillator, RB7:6 are digital IO)
	__config _FCMEN_OFF & _IESO_OFF & _BOD_OFF & _PWRTE_ON & _INTOSCIO & _CP_OFF & _CPD_OFF


; Set the device identification
;    {0x06, 0x09, 0x0, DEVICE_TYPE[3:0], 0x0, FIRMWARE_VERSION[3:0]}
	__idlocs	0x6900 | (HRMI_DEV_TYPE << 4) | HRMI_VERSION


; ------------------------------------------------------------------------
; Program Constants
;

; PIC Internal register equates
W		equ	.0
F		equ	.1
; Bank 0 and registers in all banks
INDF		equ	0x0
TMR0		equ	0x1
PCL		equ	0x2
STATUS		equ	0x3
FSR		equ	0x4
PORT_A		equ	0x5
PORT_B		equ	0x6
PORT_C		equ	0x7
PORT_E		equ	0x9
PCLATH		equ	0xA
INTCON		equ	0xB
PIR1		equ	0xC
PIR2		equ	0xD
TMR1L		equ	0xE
TMR1H		equ	0xF
T1CON		equ	0x10
TMR2		equ	0x11
T2CON		equ	0x12
SSPBUF		equ	0x13
SSPCON		equ	0x14
CCPR1L		equ	0x15
CCPR1H		equ	0x16
CCP1CON		equ	0x17
RCSTA		equ	0x18
TXREG		equ	0x19
RCREG		equ	0x1A
ADRESH		equ	0x1E
ADCON0		equ	0x1F
; Bank 1 specific
OPTION_REG	equ	0x81
TRISA		equ	0x85
TRISB		equ	0x86
TRISC		equ	0x87
TRISE		equ	0x89
PIE1		equ	0x8C
PIE2		equ	0x8D
PCON		equ	0x8E
OSCCON		equ	0x8F
OSCTUNE		equ	0x90
ANSEL		equ	0x91
PR2		equ	0x92
SSPADD		equ	0x93
SSPSTAT		equ	0x94
WPUB		equ	0x95
IOCB		equ	0x96
CMCON1		equ	0x97
TXSTA		equ	0x98
SPBRG		equ	0x99
CMCON0		equ	0x9C
VRCON		equ	0x9D
ADRESL		equ	0x9E
ADCON1		equ	0x9F
; Bank 2 specific
WDTCON		equ	0x105
LCDCON		equ	0x107
LCDPS		equ	0x108
LVDCON		equ	0x109
EEDATA		equ	0x10C
EEADR		equ	0x10D
EEDATH		equ	0x10E
EEADRH		equ	0x10F
LCDDATA0	equ	0x110
LCDDATA1	equ	0x111
LCDDATA3	equ	0x113
LCDDATA4	equ	0x114
LCDDATA6	equ	0x116
LCDDATA7	equ	0x117
LCDDATA9	equ	0x119
LCDDATA10	equ	0x11A
LCDSE0		equ	0x11C
LCDSE1		equ	0x11D
; Bank 3 specific
EECON1		equ	0x18C
EECON2		equ	0x18D
;
; PIC Internal STATUS register bits
CARRY		equ	.0
DC		equ	.1
Z		equ	.2
PD		equ	.3
TO		equ	.4
RP0		equ	.5
RP1		equ	.6
IRP		equ	.7
;
; Interrupt sources in the INTCON register
RBIF_INT	equ	.0
INTF_INT	equ	.1
TMR0_INT	equ	.2
;
; Interrupt enable bits in the INTCON register
RBIE		equ	.3
INTE		equ	.4
T0IE		equ	.5
PEIE		equ	.6
GIE		equ	.7
;
; Interrupt sources in the PIR1 register
TMR1IF		equ	.0
TMR2IF		equ	.1
CCP1IF		equ	.2
SSPIF		equ	.3
TXIF		equ	.4
RCIF		equ	.5
ADIF		equ	.6
PSPIF		equ	.7
;
; Interrupt sources in the PIR2 register
LVDIF		equ	.2
LCDIF		equ	.4
C1IF		equ	.5
C2IF		equ	.6
OSFIF		equ	.7
;
; T1CON bits
TMR1ON		equ	.0
TMR1CS		equ	.1
T1SYNC		equ	.2
T1OSCEN		equ	.3
T1CKPS0		equ	.4
T1CKPS1		equ	.5
T1GE		equ	.6
T1GINV		equ	.7
;
; T2CON bits
T2CKPS0		equ	.0
T2CKPS1		equ	.1
TMR2ON		equ	.2
TOUTPS0		equ	.3
TOUTPS1		equ	.4
TOUTPS2		equ	.5
TOUTPS3		equ	.6
;
; SSPCON bits
SSPM0		equ	.0
SSPM1		equ	.1
SSPM2		equ	.2
SSPM3		equ	.3
CKP		equ	.4
SSPEN		equ	.5
SSPOV		equ	.6
WCOL		equ	.7
;
; CCP1CON bits
CCP1M0		equ	.0
CCP1M1		equ	.1
CCP1M2		equ	.2
CCP1M3		equ	.3
CCP1Y		equ	.4
CCP1X		equ	.5
;
; RCSTA bits
RX9D		equ	.0
OERR		equ	.1
FERR		equ	.2
ADDEN		equ	.3
CREN		equ	.4
SREN		equ	.5
RX9		equ	.6
SPEN		equ	.7
;
; Status/Control bits in the ADCON0
ADC_ON		equ	.0
ADC_GO		equ	.1
CHS0		equ	.2
CHS1		equ	.3
CHS2		equ	.4
VCFG0		equ	.5
VCFG1		equ	.6
ADFM		equ	.7
;
; OPTION bits
PS0		equ	.0
PS1		equ	.1
PS2		equ	.2
PSA		equ	.3
T0SE		equ	.4
T0CS		equ	.5
INTEDG		equ	.6
RBPU		equ	.7
;
; PIE1 bits
TMR1IE		equ	.0
TMR2IE		equ	.1
CCP1IE		equ	.2
SSPIE		equ	.3
TXIE		equ	.4
RCIE		equ	.5
ADIE		equ	.6
EEIE		equ	.7
;
; PIE2 bits
LVDIE		equ	.2
LCDIE		equ	.4
C1IE		equ	.5
C2IE		equ	.6
OSFIE		equ	.7
;
; PCON bits
BOR		equ	.0
POR		equ	.1
SBOREN		equ	.4
;
; Status/Control bits in the OSCCON
IRCF2		equ	.6
IRCF1		equ	.5
IRCF0		equ	.4
OSTS		equ	.3
HTS		equ	.2
LTS		equ	.1
SCS		equ	.0
;
; OSCTUNE bits
TUN0		equ	.0
TUN1		equ	.1
TUN2		equ	.2
TUN3		equ	.3
TUN4		equ	.4
;
; ANSEL bits
AN0		equ	.0
AN1		equ	.1
AN2		equ	.2
AN3		equ	.3
AN4		equ	.4
;
; SSPSTAT bits
BF		equ	.0
UA		equ	.1
R_W		equ	.2
I2C_START	equ	.3
I2C_STOP	equ	.4
D_A		equ	.5
CKE		equ	.6
SMP		equ	.7
;
; WPUB bits
WPUB0		equ	.0
WPUB1		equ	.1
WPUB2		equ	.2
WPUB3		equ	.3
WPUB4		equ	.4
WPUB5		equ	.5
WPUB6		equ	.6
WPUB7		equ	.7
;
; WPU bits
WPU0		equ	.0
WPU1		equ	.1
WPU2		equ	.2
WPU3		equ	.3
WPU4		equ	.4
WPU5		equ	.5
WPU6		equ	.6
WPU7		equ	.7
;
; IOCB bits
IOCB4		equ	.4
IOCB5		equ	.5
IOCB6		equ	.6
IOCB7		equ	.7
;
; IOC bits
IOC4		equ	.4
IOC5		equ	.5
IOC6		equ	.6
IOC7		equ	.7
;
; CMCON1 bits
C2SYNC		equ	.0
T1GSS		equ	.1
;
; TXSTA bits
TX9D		equ	.0
TRMT		equ	.1
BRGH		equ	.2
SYNC		equ	.4
TXEN		equ	.5
TX9		equ	.6
CSRC		equ	.7
;
; CMCON0 bits
CM0		equ	.0
CM1		equ	.1
CM2		equ	.2
CIS		equ	.3
C1INV		equ	.4
C2INV		equ	.5
C1OUT		equ	.6
C2OUT		equ	.7
;
; VRCON bits
VR0		equ	.0
VR1		equ	.1
VR2		equ	.2
VR3		equ	.3
VRR		equ	.5
VREN		equ	.7
;
; ADCON1 bits
ADCS0		equ	.4
ADCS1		equ	.5
ADCS2		equ	.6
;
; WDTCON bits
SWDTE		equ	.0
WDTPS0		equ	.1
WDTPS1		equ	.1
WDTPS2		equ	.2
WDTPS3		equ	.3
;
; LCDCON bits
LMUX0		equ	.0
LMUX1		equ	.1
CS0		equ	.2
CS1		equ	.3
VLCDEN		equ	.4
WERR		equ	.5
SLPEN		equ	.6
LCDEN		equ	.7
;
; LCDPS bits
LP0		equ	.0
LP1		equ	.1
LP2		equ	.2
LP3		equ	.3
WA		equ	.4
LCDA		equ	.5
BIASMD		equ	.6
WFT		equ	.7
;
; LVDCON bits
LVDL0		equ	.0
LVDL1		equ	.1
LVDL2		equ	.2
LVDEN		equ	.4
IRVST		equ	.5
;
; LCDDAT0 bits
S0C0		equ	.0
S1C0		equ	.1
S2C0		equ	.2
S3C0		equ	.3
S4C0		equ	.4
S5C0		equ	.5
S6C0		equ	.6
S7C0		equ	.7
;
; LCDDAT1 bits
S8C0		equ	.0
S9C0		equ	.1
S10C0		equ	.2
S11C0		equ	.3
S12C0		equ	.4
S13C0		equ	.5
S14C0		equ	.6
S15C0		equ	.7
;
; LCDDAT3 bits
S0C1		equ	.0
S1C1		equ	.1
S2C1		equ	.2
S3C1		equ	.3
S4C1		equ	.4
S5C1		equ	.5
S6C1		equ	.6
S7C1		equ	.7
;
; LCDDAT4 bits
S8C1		equ	.0
S9C1		equ	.1
S10C1		equ	.2
S11C1		equ	.3
S12C1		equ	.4
S13C1		equ	.5
S14C1		equ	.6
S15C1		equ	.7
;
; LCDDAT9 bits
S0C3		equ	.0
S1C3		equ	.1
S2C3		equ	.2
S3C3		equ	.3
S4C3		equ	.4
S5C3		equ	.5
S6C3		equ	.6
S7C3		equ	.7
;
; LCDDAT10 bits
S8C3		equ	.0
S9C3		equ	.1
S10C3		equ	.2
S11C3		equ	.3
S12C3		equ	.4
S13C3		equ	.5
S14C3		equ	.6
S15C3		equ	.7
;
; LCDSE0 bits
SE0		equ	.0
SE1		equ	.1
SE2		equ	.2
SE3		equ	.3
SE4		equ	.4
SE5		equ	.5
SE6		equ	.6
SE7		equ	.7
;
; LCDSE1 bits
SE8		equ	.0
SE9		equ	.1
SE10		equ	.2
SE11		equ	.3
SE12		equ	.4
SE13		equ	.5
SE14		equ	.6
SE15		equ	.7
;
; EECON1 control bits
RD		equ	.0
WR		equ	.1
WREN		equ	.2
WRERR		equ	.3
EEPGD		equ	.7
;
; IO PORT bits
AIN0		equ	.0
AIN1		equ	.1
AIN2		equ	.2
AIN3		equ	.3
IF_TYPE		equ	.4	; 0 = I2C, 1 = Serial
EXT_RESET	equ	.5
HR_INPUT	equ	.0
CONFIG0		equ	.1
CONFIG1		equ	.2
CONFIG2		equ	.3
CONFIG3		equ	.4
CONFIG4		equ	.5
CONFIG5		equ	.6
CONFIG6		equ	.7
UTIL0		equ	.0
UTIL1		equ	.1
UTIL2		equ	.2
UTIL3		equ	.3
UTIL4		equ	.4
STATUS_LED	equ	.5
;
; HrFlags and/or IsrFlags variable bits
LostSignal	equ	.0	; Set when the mSec timer overflows indicating no signal in a valid period
SawHRedge	equ	.1	; Set when a low-to-high transition on the HR input is detected
				;  and the timer value has been saved
IncSec		equ	.2	; Set when the SubSecondTimer counts down
SawEdge1	equ	.3	; Set after we see the first edge when looking for a (new) pulse
				;  series
RewaitingForSignal equ	.4	; Set when we have lost the signal and are waiting a bit
SetCurHr	equ	.7	; Set when CurHr is loaded with a new value
;
; ISR_FLAGS_MASK indicates which Flags bit originate in the ISR
ISR_FLAGS_MASK	equ	0x07
;
; Flags variable bits
SerialIfType	equ	.0	; Set for a Serial IF, clear for I2C
RspFifoBusy	equ	.1	; Set when the RspFifo has been loaded, clear when emptied
MeasHr		equ	.3	; Set when measuring heart rate, clear when looking for HR pulses
;
; RcvReset variable bits
TrigReset	equ	.0	; Set to trigger a RMCM01 reset
ResetInProcess	equ	.1	; Set while the reset signal is asserted
ResetDone	equ	.2	; Set after a reset has been performed to prevent unnecessary
				;   resets after that
;
; SifRxState variable bits
;  SifRxIdle, SifRxProcArg, SifRxValidCmd, SifRxWaitForTerm are mutually exclusive state bits
SifRxIdle		equ	.0	; Set while waiting for a CommandChar
SifRxProcArg		equ	.1	; Set while receiving numeric argument characters
SifRxValidCmd		equ	.2	; Set when a valid command & optional arg received
SifRxWaitForTerm	equ	.3	; Set when an invalid command received to wait for end
RxHasArg		equ	.4	; Set when the CommandChar is followed by an argument
					;   (Also set by the I2C machine)
SIF_RX_STATE_MASK	equ	0x0F	; Mask of mutually exclusive state bits
;
; SifTxState variable bits
SifTxIdle	equ	.0	; Set when the SifTx machine is between arguments
SifTxOutputM	equ	.1	; Set when there is a non-zero thousands digit to transmit
SifTxOutputH	equ	.2	; Set when there is a non-zero hundreds digit to transmit
SifTxOutputT	equ	.3	; Set when there is a non-zero tens digit to transmit
SifTxOutputO	equ	.4	; Set when there is a units digit to transmit
;
; I2cState variable bits
I2cIdle		equ	.0	; Set when the I2C machine is between operations
				;   (has received or transmitted all necessary data)
I2cRxCmd	equ	.1	; Set when the I2C machine has received a W address and
				;   is waiting for the command
I2cRxArg	equ	.2	; Set when the I2C machine has received a command that
				;   requires an argument and is waiting for the arg
I2cTxData	equ	.3	; Set when the I2C machine has received a R address and
				;   is in the processing of sending data in the RspFifo
I2C_STATE_MASK	equ	0x0F	; Mask of mutually exclusive state bits
;
; I2cFlags variable bits
I2cReadExpected	equ	.0	; Set after the I2C write for commands that will generate
				;   a response
I2cReadWaiting	equ	.1	; Set by the I2C ISR code when a read is initiated but data
				;   from the command processing is not yet available
;
; Bank 0 Variable register equates
;
;
; Operating registers
TempData		equ	0x20	; Scratch register
TempData2		equ	0x21	; Scratch register
IsrFlags		equ	0x22	; Flags originating in the ISR, atomically copied to HrFlags
HrFlags			equ	0x23	; HR related flags for use in the main program
Flags			equ	0x24	; Flags for use in the main program
RcvReset		equ	0x25	; Receiver reset state
Mode			equ	0x26	; Current program operating state
MsecTimerHi		equ	0x27	; mSec timer 15:8, reset every valid low-to-high HR transition
MsecTimerLo		equ	0x28	; mSec timer  7:0
IsrTimerHi		equ	0x29	; ISR latched mSec timer 15:8 on a valid low-to-high HR transition
IsrTimerLo		equ	0x2A	; ISR latched mSec timer  7:0
CurTimerHi		equ	0x2B	; Captured mSec timer 15:8 on a valid low-to-high HR transition
					;   Set atomically from latched value in main routine
CurTimerLo		equ	0x2C	; Captured mSec timer  7:0 
RewaitingTimer		equ	0x2D
SubSecondTimerHi	equ	0x2E	; Decrements every mSec in ISR
SubSecondTimerLo	equ	0x2F	; Decrements every mSec in ISR
HeartbeatCount		equ	0x30	; Incremented in immediate mode for every heartbeat
					;   pushed into the HrHistBuf, incremented every second
					;   in averaged mode
SifRxCurChar		equ	0x31	; Current received character
SifRxState		equ	0x32	; Serial Input Processor State
SifRxArgCharCount	equ	0x33	; Serial Input Processor count of incoming argument chars
I2cState		equ	0x34	; I2C Input/Output Processor State
I2cSspBits		equ	0x35	; I2C important bits from SSPSTAT
I2cFlags		equ	0x36	; I2C read processing flags
RxCurCmd		equ	0x37	; Input Processor current validated CommandChar
RxCurArg		equ	0x38	; Input Processor current argument value
CmdPushPtr		equ	0x39	; Current free location in CmdFifo
CmdPopPtr		equ	0x3A	; Current location to get data from in CmdFifo
CmdFifoCount		equ	0x3B	; Number of valid entries currently in CmdFifo
RspFifoCount		equ	0x3C	; RspFifo current valid count
RspFifoPtr		equ	0x3D	; RspFifo current output character
SifTxState		equ	0x3E	; Serial Output Processor State
D0			equ	0x3F	; Contains the BCD units value
D1			equ	0x40	; Contains the BCD tens value
D2			equ	0x41	; Contains the BCD hundreds value
D3			equ	0x42	; Contains the BCD thousands value
;
; Subroutine temporary variables
;
PayloadCount	equ	0x50	; Counter for all data bytes in a response packet
;
; Flash access variables
FlashAddr	equ	0x55	; Address into data EEPROM
FlashData	equ	0x56	; Data to read from or write to in data EEPROM
;
; Math routine
ACCaHI		equ	0x57	; Operand 1 hi
ACCaLO		equ	0x58	; Operand 1 lo
ACCbHI		equ	0x59	; Operand 2 hi / Result hi
ACCbLO		equ	0x5A	; Operand 2 lo / Result lo
ACCcHI		equ	0x5B	; Result hi
ACCcLO		equ	0x5C	; Result lo 
ACCdHI		equ	0x5D	; Temporary operand
ACCdLO		equ	0x5E	; Temporary operand
ACCtemp		equ	0x5F
;
; ISR routine variables
IsrTempData	equ	0x7B
;
; Interrupt state save registers
FSR_Temp	equ	0x7C
STATUS_Temp	equ	0x7D
PCLATH_Temp	equ	0x7E
W_Temp		equ	0x7F
;
;
; Bank 1 Variable register equates
;
; Command Buffer: Stores validated command bytes followed by numeric
; argument bytes if they exist for the command.
;   May be up to 16 entries
CmdFifo		equ	0xE0
;
;
; Bank 2 Variable register equates
;
; Response Buffer
;   Contains data for one response
;   Up to 48 bytes
RspFifo		equ	0x20
;
; Heart rate History buffer
;   Contains a history of heart rate interval data in pairs {HI:LO}
;   Up to 32 bytes (16 entries)
HrHistBuf	equ	0x50
;
;
; Initialization constants
STATUS_SETUP	equ	B'00011000'	; 7: IRP = 0 (page 0-1)
					; 6:5: RP1:0 = 00b (page 0)
					; 4: TO = 1 (default state)
					; 3: PD = 1 (default state)
					; 2: Z = 0
					; 1: DC = 0
					; 0: C = 0

OPTION_SETUP	equ	B'01000111'	; 7: RBPU = 0 (enable pull-ups)
					; 6: INTEDG = 1 (Int on rising edge RB0/INT)
					; 5: T0CS = 0 (internal TMR0 clock)
					; 4: T0SE = 0 (rising TMR0 clock)
					; 3: PSA = 0 (prescaler to TMR0)
					; 2:0: prescaler = 111b (1:256) (TMR0 inc 128 uSec)

INTCON_SETUP	equ	B'11010000'	; 7: GIE = 1 (interrupts enabled)
					; 6: PEIE = 1 (Peripheral ints enabled)
					; 5: T0IE = 0 (TMR0 ints disabled)
					; 4: INTE = 1 (INT ints enabled)
					; 3: RBIE = 0 (RB ints disabled)
					; 2: T0IF = 0
					; 1: INTF = 0
					; 0: RBIF = 0

PIE1_SETUP	equ	B'00000010'	; 7: EEIE = 0 (EE Write Complete)
					; 6: ADIE = 0 (A/D converter)
					; 5: RCIE = 0 (USART RX Enable)
					; 4: TXIE = 0 (USART TX Enable)
					; 3: SSPIE = 0 (Sync Serial Port)
					; 2: CCP1IE = 0 (CCPQ Int enable)
					; 1: TMR2IE = 1 (TMR2 to PR2 match)
					; 0: TMR1IE = 0 (TMR1 Overflow)

PIE2_SETUP	equ	B'00000000'	; 7: OSFIE = 0 (Oscillator Fail)
					; 6: C2IE = 0 (Comparator C2)
					; 5: C1IE = 0 (Comparator C1)
					; 4: LCDIE = 0 (LCD Module)
					; 3: Unimplemented
					; 2: LVDIE = 0 (Low Voltage Detect)
					; 1: Unimplemented
					; 0: CCP2IE = 0 (CCP2 Int Enable)

; Timer 1 increments every 4 uSec
T1CON_INIT	equ	B'00110000'	; 7: T1GINV = 0
					; 6: TMR1GE = 0 (Timer1 always counts when on)
					; 5:4: T1CKPS[1:0] = 11 (1:8 prescale value)
					; 3: T1OSCEN = 0
					; 2: T1SYNC = 0
					; 1: TMR1CS = 0 (Internal clock Fosc/4)
					; 0: TMR1ON = 0 (Off)

T2CON_INIT	equ	B'00001101'	; 7: Reserved
					; 6:3: TOUTPS = 0001 (1:2 Postscale)
					; 2: TMR2ON = 1 (Timer2 is on)
					; 1:0: T2CKPS = 01 (Prescaler is 4: inc every 2 uSec)

PORTA_INIT	equ	B'00100000'	; RMCM01 reset in default state
PORTB_INIT	equ	B'00000000'	; No outputs enabled
PORTC_INIT	equ	B'00100000'	; Status LED disabled
PORTA_OE	equ	B'00111111'	; RA[7:6] outputs, RA5 set to output when needed
PORTB_OE	equ	B'11111111'	; All inputs
PORTC_OE	equ	B'11011111'	; RC[7,6], RC[4:0] configured as inputs
					;  they will be correctly configured after PIC init

OSCCON_INIT	equ	B'01110000'	; 7: Unimplemented
					; 6:4: IRCF = 111 (8 MHz)
					; 3: OSTS = 0 (Device running from int osc)
					; 2: HTS = 0
					; 1: LTS = 0
					; 0: SCS = 0 (Clock source defined by FOSC[2:0])

WPUB_INIT	equ	B'11111111'	; Enable pull-ups

IOCB_INIT	equ	B'00000000'	; Disable interrupt on change

TXSTA_INIT	equ	B'00100100'	; 7: CSRC = 0 (Don't care)
					; 6: TX9 = 0 (8-bit transmission)
					; 5: TXEN = 1 (transmit enabled)
					; 4: SYNC = 0 (asynch mode)
					; 3: Reserved
					; 2: BRGH = 1 (high speed mode)
					; 1: TRMT = 0
					; 0: TX9D = 0 (9th xmit data)

RCSTA_INIT	equ	B'00010000'	; 7: SPEN = 0 (Serial port disabled)
					; 6: RX9 = 0 (8-bit reception)
					; 5: SREN = 0 (Don't care)
					; 4: CREN = 1 (enable continuous)
					; 3: ADDEN = 0 (disable addr detection)
					; 2: FERR = 0
					; 1: OERR = 0
					; 0: RX9D = 0

ADCON0_INIT	equ	B'10000001'	; 7: ADFM = 1 (Right justify results)
					; 6: VCFG1 = 0 (Vref- = VSS)
					; 5: VCFG0 = 0 (Vref+ = VDD)
					; 4:2: CHS[2:0] = 000 (Analog Channel = 0)
					; 1: GO/~DONE = 0
					; 0: ADON = 1 (ADC on)

ADCON1_INIT	equ	B'00100000'	; 7: Unimplemented
					; 6:4: ADCS[2:0] = 010 (Fosc/32)
					; 3:0: Unimplemented

ANSEL_INIT	equ	B'00001111'	; Enable RA[3:0] as analog 

CMCON0_INIT	equ	B'00000111' 	; 7: C2OUT = 0
					; 6: C1OUT
					; 5: C2INV
					; 4: C1INV
					; 3: CIS
					; 2:0: CM2:0 = 111 (Comparators off, CxIN pins digital IO)

PR2_INIT	equ	.249		; 250 x 4 uSec inc value = Timer2 int every 1 mSec

SSPSTAT_INIT	equ	B'00000000'	; 7: SMP = 0
					; 6: CKE = 0
					; 5: D/A = 0
					; 4: P = 0
					; 3: START = 0
					; 2: R/W = 0
					; 1: UA = 0
					; 0: BF = 0

SSPCON_INIT	equ	B'00010110'	; 7: WCOL = 0
					; 6: SSPOV = 0
					; 5: SSPEN = 0 (Synch serial port enable)
					; 4: CKP = 1 (clock idle is low state)
					; 3:0: SSPM3:0 = 0110 (I2C Slave mode, 7-bit address)

LCDCON_INIT	equ	B'01000000'	; 7: LCDEN = 0 (LCD driver disabled)
					; 6: nSLPEN = 1 (LCD driver in sleep disabled)
					; 5: WERR = 0
					; 4: VLCDEN = 0 (VLCD pins disabled)
					; 3:2: CS1:0 = 00 (Fosc/8192)
					; 1:0: LMUX1:0 = 00 (Static bias)

; Baud rate constants
SPBRG_2_4		equ	.207	; Baud rate =  1200 @ 8 MHz, BRGH = 1
SPBRG_4_8		equ	.103	; Baud rate =  4800 @ 8 MHz, BRGH = 1
SPBRG_9_6		equ	.51	; Baud rate =  9600 @ 8 MHz, BRGH = 1
SPBRG_19_2		equ	.25	; Baud rate = 19200 @ 8 MHz, BRGH = 1
SPBRG_38_4		equ	.12	; Baud rate = 38400 @ 8 MHz, BRGH = 1
SPBRG_57_6		equ	.8	; Baud rate = 57600 @ 8 MHz, BRGH = 1

; Global constants
HRMI_DEV_TYPE		equ	0x02		; Device type for this firmware
HRMI_VERSION		equ	0x01		; Firmware version
RMCM01_RESET_DELAY	equ	.256-.128	; RMCM01 reset period: 128x128 uS = 10.24 mS
REWAIT_TIME		equ	.15		; Number of second intervals we wait after loss-of-signal
						;   before resetting the buffers and storing a 0 value
MAX_HR_HIST		equ	.32		; Number of bytes in the heartrate history buffer
MAX_REQ_HR_VALS		equ	.16		; Maximum number of history buffer entries that can be read
SS_INIT_HI		equ	0x03		; SubSecond Timer initialization value
SS_INIT_LO		equ	0xE8		;   Set to 1000 (number of mSec ISR calls/sec)
ST_HR_INIT		equ	.65536-.25000	; 100 mSec heart rate status blink period
CMD_FIFO_DEPTH		equ	.16		; Number of entries in CmdFifo
CMD_FIFO_MASK		equ	0x0F		; Mask of valid bits in CmdFifo pointers
NUM_ADC_PORTS		equ	.4		; Maximum number of ADC ports that can be read
UTIL_PORT_MASK		equ	0x1F		; Mask of valid utility port bits
;
; Command Packet encodings
CMD_READ_ADC		equ	'A'
CMD_SET_IO_DIR		equ	'D'
CMD_GET_HR		equ	'G'
CMD_GET_IO_PINS		equ	'I'
CMD_GET_MODE		equ	'M'
CMD_SET_IO_PINS		equ	'O'
CMD_SET_MODE		equ	'S'
CMD_GET_VERSION		equ	'V'
CMD_TERM_CHAR		equ	0x0D		; <CR> character
;
; Command argument flags - return valus for CheckValidCommand
;   bit 0: Set to indicate a valid command
;   bit 1: Set to indicate the command requires a subsequent argument
;   bit 2: Set to indicate the command returns a response
;
CMD_READ_ADC_ARG	equ	B'00000111'
CMD_SET_IO_DIR_ARG	equ	B'00000011'
CMD_GET_HR_ARG		equ	B'00000111'
CMD_GET_IO_PINS_ARG	equ	B'00000101'
CMD_GET_MODE_ARG	equ	B'00000101'
CMD_SET_IO_PINS_ARG	equ	B'00000011'
CMD_SET_MODE_ARG	equ	B'00000011'
CMD_GET_VERSION_ARG	equ	B'00000101'
;
; HrDataRsp flags 
HR_FLAGS_NO_SIG		equ	.1		; No Data flag


; ------------------------------------------------------------------------
; Macro Definitions
;
ASSERT_RMCM01_RESET	macro
	bcf		PORT_A,EXT_RESET
	bsf		STATUS,RP0
	bcf		TRISA,EXT_RESET		; Make the pin an output
	bcf		STATUS,RP0
	endm

DEASSERT_RMCM01_RESET	macro
	bsf		STATUS,RP0
	bsf		TRISA,EXT_RESET		; Make the pin an input
	bcf		STATUS,RP0
	bsf		PORT_A,EXT_RESET
	endm

SET_STATUS_OFF		macro
	bsf		PORT_C,STATUS_LED
	endm

SET_STATUS_ON		macro
	bcf		PORT_C,STATUS_LED
	endm

SET_STATUS_TMR		macro StDelayVal
	movlw		StDelayVal & 0xFF
	movwf		TMR1L
	movlw		(StDelayVal >> 8) & 0xFF
	movwf		TMR1H
	bsf		T1CON,TMR1ON
	bcf		PIR1,TMR1IF
	endm


SET_SIF_RX_ST		macro	StateBit
	movf		SifRxState,W
	andlw		~SIF_RX_STATE_MASK
	movwf		SifRxState
	bsf		SifRxState,StateBit
	endm

SET_I2C_ST		macro	StateBit
	movf		I2cState,W
	andlw		~I2C_STATE_MASK
	movwf		I2cState
	bsf		I2cState,StateBit
	endm

	; SET_RSP_FIFO_BUSY atomically loads the value in W into RspFifoCount,
	; and sets the Busy flag to prevent a race condition with the I2C
	; ISR code which resets the RspFifo when it loads a command.  Trigger
	; the SSP module with the first byte of data in the RspFifo if the I2C code
	; has already seen the start of the Read.
SET_RSP_FIFO_BUSY	macro
	bcf		INTCON,GIE	; Disable interrupts
	movwf		RspFifoCount	; Set number of values in RspFifo
	bsf		Flags,RspFifoBusy
	btfsc		I2cFlags,I2cReadWaiting
	call		LoadFirstI2cReadData
	bsf		INTCON,GIE	; Re-enable interrupts
	endm



; ------------------------------------------------------------------------
; Data EEPROM Initialization
;
	org		0x2100

; Default IO Pin direction
;    Bits 4:0: (0 = Output, 1 = Input; default: All outputs)
;
EEP_IO_DIR_ADDR		equ	.0
EEP_IO_DIR_VAL		de	0x00

; Default IO Pin value
;    Bits 4:0: (default: All 0)
;
EEP_IO_VAL_ADDR		equ	EEP_IO_DIR_ADDR+.1
EEP_IO_VAL_VAL		de	0x00



; ------------------------------------------------------------------------
; Code Block
; 
	org		0x0

	; Reset Vector
	goto		MAIN_VECTOR


; ------------------------------------------------------------------------
; Interrupt Code Block
; Handles interrupts from the following sources:
;   1. SSPIF for the I2C interface when enabled
;   2. RCIF for the serial interface when enabled
;   3. TIMER2 for the mSec Timer
;   4. INTF for a change on the RCMC01 HRinput signal
;   
	org		0x4
	; Save interrupt state
	movwf		W_Temp
	swapf		STATUS,W
	clrf		STATUS
	movwf		STATUS_Temp
	movf		PCLATH,W
	movwf		PCLATH_Temp
	clrf		PCLATH
	movf		FSR,W
	movwf		FSR_Temp

	; Check I2C interrupt first because it is the most timing critical
	btfsc		Flags,SerialIfType	; Only check this interrupt if enabled
	goto		ISR_CHECK_SIF_INT
	btfss		PIR1,SSPIF		; Sync Serial Port interrupt?
	goto		ISR_IF_RET		;    N: Check next interrupt
	bcf		PIR1,SSPIF
	pagesel		HANDLE_I2C_INT
	goto		HANDLE_I2C_INT		;    Y: Process the I2C interrupt
						;       (continues at ISR_IF_RET)

ISR_CHECK_SIF_INT
	btfss		PIR1,RCIF		; USART receive buffer full?
	goto		ISR_IF_RET		;    N: Check next interrupt
	pagesel		HANDLE_SIF_CHAR
	goto		HANDLE_SIF_CHAR		;    Y: Process the character
						;       (continues at ISR_IF_RET)

ISR_IF_RET
	; Check for TIMER2 interrupt
	btfss		PIR1,TMR2IF		; TMR2 interrupting?
	goto		ISR_CHECK_HR_INT	;    N: Check for HR input interrupt
	bcf		PIR1,TMR2IF		;    Y: Reset interrupt and process timer

	; Increment mSec timer
	incf		MsecTimerLo,F
	btfsc		STATUS,Z		; Lo half rollover?
	incf		MsecTimerHi,F		;    Y: increment high half
	; Check if the timer has exceeded a realistic period for a minimum heart rate
	; (and practical for code execution speed).  We look for a period in excess of
	; 1792 mSec (0x0700 mSec for 33.48 bpm) because we can easily check for MSByte >= 0x07
	movf		MsecTimerHi,W
	sublw		0x07			; W = MsecTimerHi - 0x07
	btfsc		STATUS,Z		; Hi half rollover?
	bsf		IsrFlags,LostSignal	;    Y: note no signal in a valid period
						;    N: continue

	; Process SubSecondTimer
	decfsz		SubSecondTimerLo,F	; --SubSecondTimerLo = 0?
	goto		ISR_CHECK_HR_INT	;    N: continue
	movf		SubSecondTimerHi,W	;    Y: Check high half
	btfsc		STATUS,Z		; High Half already 0?
	goto		ISR_SECOND_MARK		;    Y: At second mark
	decf		SubSecondTimerHi,F	;    N: --SubSecondTimerHi
	goto		ISR_CHECK_HR_INT

ISR_SECOND_MARK
	; Note on-second
	bsf		IsrFlags,IncSec
	; Reset sub-second timer
	movlw		SS_INIT_HI
	movwf		SubSecondTimerHi
	movlw		SS_INIT_LO
	movwf		SubSecondTimerLo

ISR_CHECK_HR_INT
	btfss		INTCON,INTF_INT		; Change on HR input?
	goto		ISR_RESTORE		;    N: Done
	movf		PORT_B,W		; Read PORT_B so we can clear the interrupt
	bcf		INTCON,INTF_INT
	btfss		PORT_B,HR_INPUT		; Sample current HR_INPUT value
	goto		ISR_RESTORE		;    Low: done with this interrupt
	; Check if this signal is within 250 mSec of the previous signal indicating non-timing
	; related information from a coded Polar transmiter.  We ignore those pulses (the
	; "window" documented in Polar's Engineering Guide).
	movf		MsecTimerHi,W
	btfss		STATUS,Z		; Hi-half 0?
	goto		ISR_VALID_HR_INT	;    N: valid pulse (> 250 mSec)
	movf		MsecTimerLo,W		;    Y: check if MsecTimer > 250
	sublw		.250			; W = 250 - MsecTimer
	btfsc		STATUS,CARRY		; C = 0 if MsecTimer > 250
	goto		ISR_RESTORE		;    C = 1: not a timing pulse

ISR_VALID_HR_INT
	; Save the current timer value
	movf		MsecTimerHi,W
	movwf		IsrTimerHi
	movf		MsecTimerLo,W
	movwf		IsrTimerLo
	; Reset the timer
	clrf		MsecTimerHi
	clrf		MsecTimerLo
	; Note pulse
	bsf		IsrFlags,SawHRedge

ISR_RESTORE
	; Restore interrupt state
	movf		FSR_Temp,W
	movwf		FSR
	movf		PCLATH_Temp,W
	movwf		PCLATH
	swapf		STATUS_Temp,W
	movwf		STATUS
	swapf		W_Temp,F
	swapf		W_Temp,W
	; Return from the interrupt (and re-enable interrupts)
	retfie


; ------------------------------------------------------------------------
; Main Code Block
; 
MAIN_VECTOR

	; Initialize unique PIC control registers in page 0
	clrf		TMR0
	clrf		FSR
	clrf		PCLATH
	movlw		STATUS_SETUP
	movwf		STATUS
	movlw		PORTA_INIT
	movwf		PORT_A
	movlw		PORTB_INIT
	movwf		PORT_B
	movlw		PORTC_INIT
	movwf		PORT_C
	movlw		T1CON_INIT
	movwf		T1CON
	movlw		SSPCON_INIT
	movwf		SSPCON
	movlw		RCSTA_INIT
	movwf		RCSTA
	movlw		ADCON0_INIT
	movwf		ADCON0

	; Initialize unique PIC control registers in page 1
	bsf		STATUS,RP0	; Point to page 1 registers
	movlw		OPTION_SETUP
	movwf		OPTION_REG
	movlw		PORTA_OE
	movwf		TRISA
	movlw		PORTB_OE
	movwf		TRISB
	movlw		PORTC_OE
	movwf		TRISC
	movlw		PIE1_SETUP
	movwf		PIE1
	movlw		PIE2_SETUP
	movwf		PIE2
	movlw		ANSEL_INIT
	movwf		ANSEL
	movlw		SSPSTAT_INIT
	movwf		SSPSTAT
	movlw		WPUB_INIT
	movwf		WPUB
	movlw		IOCB_INIT
	movwf		IOCB
	movlw		TXSTA_INIT
	movwf		TXSTA
	movlw		CMCON0_INIT
	movwf		CMCON0
	movlw		ADCON1_INIT
	movwf		ADCON1
	bcf		STATUS,RP0	; Point to page 0 registers again

	; Initialize unique PIC control registers in page 2
	bsf		STATUS,RP1	; Point to page 2 registers
	movlw		LCDCON_INIT
	movwf		LCDCON
	clrf		EEADRH
	clrf		EEDATH
	bcf		STATUS,RP1	; Point to page 0 registers again

	; Initialize the built-in oscillator
	bsf		STATUS,RP0
	movlw		OSCCON_INIT
	movwf		OSCCON
OSC_INIT_LOOP
	; Spin until the internal oscillator is stable
	btfss		OSCCON,HTS
	goto		OSC_INIT_LOOP
	bcf		STATUS,RP0

	; Initialize variables
	clrf		IsrFlags
	clrf		HrFlags
	clrf		Flags
	clrf		RcvReset
	clrf		MsecTimerHi
	clrf		MsecTimerLo
	movlw		SS_INIT_HI
	movwf		SubSecondTimerHi
	movlw		SS_INIT_LO
	movwf		SubSecondTimerLo
	clrf		HeartbeatCount
	clrf		SifRxState
	bsf		SifRxState,SifRxIdle
	clrf		I2cState
	bsf		I2cState,I2cIdle
	clrf		I2cFlags
	clrf		CmdPushPtr
	clrf		CmdPopPtr
	clrf		CmdFifoCount
	clrf		RspFifoCount
	clrf		RspFifoPtr
	clrf		SifTxState
	bsf		SifTxState,SifTxIdle

	; Load all zeros as initial values in the heartrate history buffer
	call		ResetBuff

	; Load the power-on averaging mode and utility port pin direction and values
	call		LoadPowerOnVals

	; Enable TIMER2 to start generating timing interrupts
	clrf		TMR2
	bsf		STATUS,RP0
	movlw		PR2_INIT
	movwf		PR2
	bcf		STATUS,RP0
	movlw		T2CON_INIT
	movwf		T2CON

	; Read the external config values
	btfss		PORT_A,IF_TYPE
	goto		INIT_I2C_IF
	; Initialize the Serial interface
	bsf		Flags,SerialIfType	; Store type
	bsf		STATUS,RP0
	call		SelectBaud
	movwf		SPBRG
	pageselw	$
	bsf		PIE1,RCIE		; Enable USART receive interrupt
	bcf		STATUS,RP0
	bsf		RCSTA,SPEN		; Enable serial port
	goto		INIT_ENABLE_INTS

INIT_I2C_IF
	; Initialize the I2C interface
	movf		PORT_B,W		; W[7:0] = {I2C address[6:0], X}
	andlw		0xFE			; Mask off low bit
	bsf		STATUS,RP0
	movwf		SSPADD			; Store the device I2C address
	bsf		PIE1,SSPIE		; Enable SSP interrupt
	bcf		STATUS,RP0
	bsf		SSPCON,SSPEN		; Enable I2C interface

INIT_ENABLE_INTS
	; Finally, enable interrupts before starting normal operation
	movlw		INTCON_SETUP
	movwf		INTCON

MAIN_LOOP
	clrwdt

	; --------------------------------
	; Process HR pulses
	;   call GetIsrHrValues
	;   if (HrFlags.SawHRedge || HrFlags.LostSignal || HrFlags.IncSec)
	;     call ProcessHrPulse
	;     if (HrFlags.SetCurHr)
	;       ++HeartbeatCount
	;       call StoreCurHr
	;       
	call		GetIsrHrValues

	btfsc		HrFlags,SawHRedge
	goto		MAIN_PROCESS_HR_PULSE
	btfsc		HrFlags,LostSignal
	goto		MAIN_PROCESS_HR_PULSE
	btfss		HrFlags,IncSec
	goto		MAIN_CHECK_RCV_RESET
MAIN_PROCESS_HR_PULSE
	call		ProcessHrPulse
	btfss		HrFlags,SetCurHr
	goto		MAIN_CHECK_RCV_RESET
	incf		HeartbeatCount,F
	call		StoreCurHr

MAIN_CHECK_RCV_RESET
	; --------------------------------
	; Process RMCM01 reset
	;   if (RcvReset.TrigReset)
	;      assert reset
	;      initialize reset timer
	;      RcvReset.TrigReset = 0
	;      RcvReset.ResetInProcess = 1
	;   else if (Rcv.ResetInProcess)
	;      if timer expired
	;          deassert reset
	;          RcvReset.ResetInProcess = 0
	;          RcvReset.ResetDone = 1
	btfss		RcvReset,TrigReset
	goto		MAIN_CHECK_RCV_RESET_1
	ASSERT_RMCM01_RESET
	movlw		RMCM01_RESET_DELAY	; Configure TMR0 for reset period
	movwf		TMR0
	bcf		INTCON,TMR0_INT
	bcf		RcvReset,TrigReset
	bsf		RcvReset,ResetInProcess
	goto		MAIN_CHECK_HB_STATUS
MAIN_CHECK_RCV_RESET_1
	btfss		RcvReset,ResetInProcess
	goto		MAIN_CHECK_HB_STATUS
	btfss		INTCON,TMR0_INT
	goto		MAIN_CHECK_HB_STATUS
	DEASSERT_RMCM01_RESET
	bcf		RcvReset,ResetInProcess
	bsf		RcvReset,ResetDone

MAIN_CHECK_HB_STATUS
	; --------------------------------
	; Manage status indicator
	;   if (!Flags.MeasHr)
	;     set status light on and clear status timer for solid on
	btfsc		Flags,MeasHr
	goto		MAIN_CHECK_HB_STATUS_1
	SET_STATUS_ON				; Set solid on
	bcf		T1CON,TMR1ON		; Clear and disable status timer
	bcf		PIR1,TMR1IF
	goto		MAIN_CHECK_CMD
MAIN_CHECK_HB_STATUS_1
	;   if (Flags.MeasHr && HrFlags.SawHRedge)
	;     set status light on and program status timer for a blink
	btfss		Flags,MeasHr
	goto		MAIN_CHECK_CMD
	btfss		HrFlags,SawHRedge
	goto		MAIN_CHECK_CMD
	SET_STATUS_ON				; Set HR pulse
	SET_STATUS_TMR	ST_HR_INIT


MAIN_CHECK_CMD
	; --------------------------------
	; Process incoming commands
	;   if (CmdFifoCount != 0)
	;      call HandleCommand
	movf		CmdFifoCount,W
	btfsc		STATUS,Z
	goto		MAIN_CHECK_SIF_TX
	pagesel		HandleCommand
	call		HandleCommand
	pagesel		$

MAIN_CHECK_SIF_TX
	; --------------------------------
	; Look if we can send a response character through the serial port if enabled
	;   if (Flags.SerialIfType && Flags.RspFifoBusy && PIR1.TXIF)
	;     call ProcessSifTx
	btfss		Flags,SerialIfType
	goto		MAIN_CHECK_STATUS_TIMER
	btfss		Flags,RspFifoBusy
	goto		MAIN_CHECK_STATUS_TIMER
	btfss		PIR1,TXIF
	goto		MAIN_CHECK_STATUS_TIMER
	pagesel		ProcessSifTx
	call		ProcessSifTx
	pagesel		$

MAIN_CHECK_STATUS_TIMER
	; --------------------------------
	; Process Status Timer
	;   if (Status timer expired)
	;      clear status LED
	btfss		PIR1,TMR1IF
	goto		MAIN_LOOP

	; Turn off timer and status LED til next time
	SET_STATUS_OFF
	bcf		T1CON,TMR1ON
	goto		MAIN_LOOP
	


; ------------------------------------------------------------------------
; Start of subroutines
;


; LoadPowerOnVals: Load power-on values from data EEPROM
;
; On entry: none
;
; On exit: Utility pins direction and value configured
;
LoadPowerOnVals
	; Setup the Utilty pin IO values before we set direction
	movlw		EEP_IO_VAL_ADDR
	movwf		FlashAddr
	call		ReadFlash
	movf		FlashData,W
	andlw		UTIL_PORT_MASK
	movwf		FlashData
	movf		PORT_C,W
	andlw		~UTIL_PORT_MASK
	iorwf		FlashData,W
	movwf		PORT_C

	; Setup the Utilty pin IO direction values
	movlw		EEP_IO_DIR_ADDR
	movwf		FlashAddr
	call		ReadFlash
	movf		FlashData,W
	andlw		UTIL_PORT_MASK
	movwf		FlashData
	bsf		STATUS,RP0
	movf		TRISC,W
	bcf		STATUS,RP0
	andlw		~UTIL_PORT_MASK
	iorwf		FlashData,W
	bsf		STATUS,RP0
	movwf		TRISC
	bcf		STATUS,RP0
	return


; SelectBaud: Return the correct USART baud rate generator initialization
; value based on the input pin strapping.
;
; on entry: Baud rate input pins strapped to desired value
;
; on exit: W contains the initialization value
;
; calling limit: callable by any routine (designed to be called by
;                routines in page 1 register space)
;
; Notes: 
;
SelectBaud
	pageselw	SELECT_BAUD_TABLE
	bcf		STATUS,RP0	; Point to page 0 registers again
	rrf		PORT_B,W	; W = {C, RB[7:1]}
	bsf		STATUS,RP0	; Point back to page 1 registers
	andlw		0x3		; Mask all but low bits
	addwf		PCL,F		; Jump into table below
SELECT_BAUD_TABLE
	retlw		SPBRG_2_4	; 00
	retlw		SPBRG_19_2	; 01
	retlw		SPBRG_38_4	; 10
	retlw		SPBRG_9_6	; 11


; ReadFlash : Read the data flash memory.  Based on the routine
; provided in the specification.
;
; on entry: FlashAddr contains the byte address to read
;
; on exit: FlashData contains the data read
;
; calling limit: callable by any routine
;
; execution time: 13 cycles
;
; Notes: none
;
ReadFlash
	movf	FlashAddr,W	; Set the EEADR reg in bank 2
	bsf	STATUS,RP1
	bcf	STATUS,RP0
	movwf	EEADR
	bsf	STATUS,RP0	; Move to bank 3
	bcf	EECON1,EEPGD	; Point to data memory
	bsf	EECON1,RD
	bcf	STATUS,RP0	; Back to bank 2
	movf	EEDATA,W	; Then copy EEDATA to FlashData
	bcf	STATUS,RP1	; Back to bank 0
	movwf	FlashData
	return


; WriteFlash : Write the data flash memory.  Based on the routine
; provided in the specification, modified not to use interrupts.  
; This routine spins until the write is done.
;
; on entry: FlashAddr contains the byte address to write
;           FlashData contains the write data
;
; on exit: none
;
; calling limit: callable by any routine
;
; Notes: 
;
WriteFlash
	movf	FlashAddr,W	; Set the EEADR reg in bank 2
	bsf	STATUS,RP1
	bcf	STATUS,RP0
	movwf	EEADR
	bcf	STATUS,RP1
	movf	FlashData,W	; Set the EEDATA reg
	bsf	STATUS,RP1
	movwf	EEDATA
	bsf	STATUS,RP0	; Move to bank 3
	bcf	EECON1,EEPGD	; Point to data memory
	bsf	EECON1,WREN	; Enable writes
	bcf	INTCON,GIE	; Disable interrupts
	movlw	0x55		; Write special sequence
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR	; Start write operation
	bsf	INTCON,GIE	; Re-enable interrupts
WriteFlashLoop
	clrwdt			; Clear WDT since it has no prescaler and worst case 
				;  flash write is longer than shortest case timeout
	btfsc	EECON1,WR	; Wait for write to finish
	goto	WriteFlashLoop
	bcf	EECON1,WREN
	bcf	STATUS,RP1	; And finally return to bank 0
	bcf	STATUS,RP0
	return


; GetIsrHrValues: Capture the output of the ISR atomically in one place for use
; by the main code
;
; On entry: IsrFlags may be updated
;           IsrTimer{Hi,Lo} may be updated
;
; On exit: Flags updated
;          CurTimer{Hi,Lo} updated
;
GetIsrHrValues
	bcf		INTCON,GIE		; Disable interrupts
	movlw		~ISR_FLAGS_MASK
	andwf		HrFlags,W		; Save all bits except what may be set from the ISR
	iorwf		IsrFlags,W		;   and then set them
	movwf		HrFlags
	movlw		~ISR_FLAGS_MASK		; Then clear those bits in the IsrFlags variable
	andwf		IsrFlags,F
	movf		IsrTimerHi,W
	movwf		CurTimerHi
	movf		IsrTimerLo,W
	movwf		CurTimerLo
	bsf		INTCON,GIE		; Re-enable interrupts
	return


; ProcessHrPulse: Process heart-rate pulse information from the ISR
;
; On entry: HrFlags set with new status
;           Flags contains current operating mode and status
;           CurTimer{Hi,LO} may be set with the current pulse delta (period)
;           RcvReset contains the current receiver reset state
;       
; On exit: Flags updated
;          RewaitingTimer may be updated
;          HrFlags.SetCurHr may be set
;          RcvReset may be updated
;          The current HR period may be pushed into the history buffer
;       
ProcessHrPulse
	bcf		HrFlags,SetCurHr	; Will be set if necessary

	; Process the Rewaiting logic (have lost the signal and are waiting a short
	; time to see if we can re-acquire it before resetting the averaging buffer)
	btfss		HrFlags,RewaitingForSignal
	goto		PROC_HR_FOR_SIGNAL_LOSS
	; Check if it's time to increment the RewaitingTimer
	btfss		HrFlags,IncSec
	goto		PROC_HR_FOR_PULSE
	incf		RewaitingTimer,F
	; Rewaiting timer timeout?
	movlw		REWAIT_TIME		; Check if it's time to clear our data
	subwf		RewaitingTimer,W
	btfss		STATUS,Z		; Have we waited REWAIT_TIME?
	goto		PROC_HR_FOR_PULSE	;    N: Check if we've seen a pulse
						;    Y: Completely re-start acquisition
	; Start a reset of the receiver to allow it to more quickly find a different
	; coded transmitter if different users will be using this device
	btfsc		RcvReset,ResetDone	; Only do a reset once per rewaiting timeout
	goto		PROC_HR_RESTART_ACQ
	btfsc		RcvReset,ResetInProcess	; Only trigger a reset if we're not currently
	goto		PROC_HR_RESTART_ACQ	;   already asserting reset
	bsf		RcvReset,TrigReset
PROC_HR_RESTART_ACQ
	call		ResetBuff
	goto		PROC_HR_FOR_PULSE


PROC_HR_FOR_SIGNAL_LOSS
	; Look for a loss of signal
	btfss		HrFlags,LostSignal	; No signal in a valid time period?
	goto		PROC_HR_FOR_PULSE	;    N: Check for pulse
						;    Y: Setup to restart acquiring a valid
						;       pulse series
	clrf		RewaitingTimer
	bsf		HrFlags,RewaitingForSignal
	bcf		HrFlags,SawEdge1
	bcf		Flags,MeasHr
	return

PROC_HR_FOR_PULSE
	; Check for and process a HR pulse
	btfss		HrFlags,SawHRedge	; Have we just seen a pulse from the receiver?
	return					;    N: Continue
	btfsc		HrFlags,SawEdge1	; Have we already seen at least one pulse?
	goto		PROC_HR_LOAD_PERIOD	;    Y: Continue processing this pulse
						;    N: Note first pulse
	bsf		HrFlags,SawEdge1
	bcf		HrFlags,RewaitingForSignal	; in case they were set
	bcf		RcvReset,ResetDone
	return

PROC_HR_LOAD_PERIOD
	bsf		Flags,MeasHr
	call		StoreCurHr
	return


; ResetBuff: Clear the history buffer
;
; On entry: none
;
; On exit: HrFlags.SawEdge1 cleared
;          Averaging and Acquisition buffers cleared
;
ResetBuff
	bcf		HrFlags,SawEdge1

	bsf		STATUS,IRP		; Hist buffer in bank 2
	movlw		MAX_HR_HIST
	movwf		TempData
	movlw		HrHistBuf
	movwf		FSR
MAIN_INIT_HR_HIST
	clrf		INDF
	incf		FSR,F
	decfsz		TempData,F
	goto		MAIN_INIT_HR_HIST
	bcf		STATUS,IRP
	return


; StoreCurHr: Push the current heartrate value into the Heartrate history buffer
;
; On entry: CurTimer{Hi,Lo} contain the current pulse period
;
; On exit: CurTimer{Hi,Lo} has been pushed into the history buffer location 0-1, all other
;            locations moved to the next location and the last locations thrown away
;          HrFlags.SetCurHr set
;
StoreCurHr
	; First make room for this entry
	bsf		STATUS,IRP
	movlw		MAX_HR_HIST-2
	movwf		TempData		; Setup loop count to move 2 less than num entries
HR_CALC_HR_HIST_LOOP
	decf		TempData,W
	addlw		HrHistBuf
	movwf		FSR			; FSR = location of current value to move
	movf		INDF,W
	movwf		TempData2
	incf		FSR,F
	incf		FSR,F			; FSR = location to put current value
	movf		TempData2,W
	movwf		INDF
	decfsz		TempData,F
	goto		HR_CALC_HR_HIST_LOOP
	; Then load the current entry
	movlw		HrHistBuf
	movwf		FSR
	movf		CurTimerHi,W
	movwf		INDF
	incf		FSR,F
	movf		CurTimerLo,W
	movwf		INDF
	bcf		STATUS,IRP
	bsf		HrFlags,SetCurHr
	return



; ------------------------------------------------------------------------
; Start of Page 1 subroutines
;
PAGE_1		org	0x0800

; HANDLE_SIF_CHAR: ISR Handler routine for incoming serial data.  This code
; segment implements the Input Processor for serial data.  It is called for
; each incoming character.  On receipt of a termination character (<CR>) it
; pushes valid commands and optional arguments into the CmdFifo.
;
; On entry: SifRxState contains the current state
;           SifRxArgCharCount contains the current number of numeric characters
;             that are part of an expected argument current received
;
; On exit: RxCurCmd may be set with a validated command
;          RxCurArg may be set with a converted numeric argument string
;          SifRxState updated
;          CmdFifo may be loaded
;
HANDLE_SIF_CHAR
	; Check the RX status for bad data
	btfsc		RCSTA,FERR		; Check for framing error
	goto		SIF_RX_BAD_RX_STATUS	;   Framing error
	btfss		RCSTA,OERR		; Check for overrun
	goto		SIF_RX_GET_RX_CHAR	;   no overrun
						;   overrun
	movf		RCREG,W			; Drain entry 1 of the SIF RX FIFO
SIF_RX_BAD_RX_STATUS
	movf		RCREG,W			; Drain entry 2 of the RX FIFO or clear
						;   badly framed character
	bcf		RCSTA,CREN		; Reset the RX port and clear errors
	bsf		RCSTA,CREN
	clrf		SifRxState		; Abandon this command sequence
	bsf		SifRxState,SifRxWaitForTerm
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

SIF_RX_GET_RX_CHAR
	; Get the character from the interface (only read it once since RCREG is a FIFO)
	movf		RCREG,W
	movwf		SifRxCurChar

	; Jump to specific SifRxState state code
	btfsc		SifRxState,SifRxIdle
	goto		SIF_RX_STATE_IDLE
	btfsc		SifRxState,SifRxProcArg
	goto		SIF_RX_STATE_PROC_ARG
	btfsc		SifRxState,SifRxValidCmd
	goto		SIF_RX_STATE_VALID_CMD
	btfsc		SifRxState,SifRxWaitForTerm
	goto		SIF_RX_STATE_WAIT_TERM
	; else there was no state bit set, so assume idle
	bsf		SifRxState,SifRxIdle

SIF_RX_STATE_IDLE
	; Idle State:
	; 
	;   if (SifRxCurChar == ValidCommandChar)
	;     RxCurCmd = SifRxCurChar
	;     if (Argument Required for this command)
	;        SifRxArgCharCount = 0
	;        RxCurArg = 0
	;        SifRxState.RxHasArg = 1
	;        SifRxState = SifRxProcArg
	;     else
	;        SifRxState.RxHasArg = 0
	;        SifRxState = SifRxValidCmd
	;   else
	;     do nothing
	;      
	movf		SifRxCurChar,W
	movwf		IsrTempData
	call		CheckValidCommand
	movwf		IsrTempData
	btfss		IsrTempData,.0			; Valid command?
	goto		SIF_RX_STATE_IDLE_NOT_CMD	;   N: Done with this command string
	movf		SifRxCurChar,W			;   Y: Set RxCurCmd
	movwf		RxCurCmd
	btfss		IsrTempData,.1			; Command has argument?
	goto		SIF_RX_STATE_IDLE_CMD_0		;   N: Setup to wait for term
	clrf		SifRxArgCharCount		;   Y: Setup to get numeric argument
	clrf		RxCurArg
	SET_SIF_RX_ST	SifRxProcArg
	bsf		SifRxState,RxHasArg
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

SIF_RX_STATE_IDLE_CMD_0
	; No subsequent argument
	SET_SIF_RX_ST	SifRxValidCmd
	bcf		SifRxState,RxHasArg
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

SIF_RX_STATE_IDLE_NOT_CMD
	; Skip all unknown characters
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

SIF_RX_STATE_PROC_ARG
	; Getting Argument State:
	;
	;   if (SifRxCurChar >= '0') && (SifRxCurChar <= '9')
	;     if (SifRxArgCharCount < 3)
	;        RxCurArg = RxCurArg*10 + (SifRxCurChar - '0')
	;        SifRxArgCharCount++
	;     else
	;        SifRxState = SifRxWaitForTerm
	;   else if (SifRxCurChar == ' ')
	;     if (SifRxArgCharCount == 0)
	;        do nothing
	;     else
	;        SifRxState = SifRxValidCmd
	;   else if (SifRxCurChar == TERM_CHAR)
	;     if (SifRxArgCharCount != 0)
	;        Try to push SifRxCmdBuf and SifRxCmdArg into CmdFifo
	;     SifRxState = SifRxIdle
	;   else
	;     SifRxState = SifRxWaitForTerm
	;
	movlw		'0'
	subwf		SifRxCurChar,W		; W = SifRxCurChar - '0'
	btfss		STATUS,CARRY		; C = 1 if SifRxCurChar >= '0'
	goto		SIF_RX_STATE_PROC_ARG_NOT_NUM
	movf		SifRxCurChar,W
	sublw		'9'			; W = '9' - SifRxCurChar
	btfss		STATUS,CARRY		; C = 1 if SifRxCurChar <= '9'
	goto		SIF_RX_STATE_PROC_ARG_NOT_NUM
	movlw		.3
	subwf		SifRxArgCharCount,W	; W = SifRxArgCharCount - 3
	btfsc		STATUS,CARRY		; C = 0 if SifRxArgCharCount < 3
	goto		SIF_RX_STATE_PROC_ARG_GT3
	call		MultCurArgByTen		; RxCurArg *= 10
	movlw		'0'
	subwf		SifRxCurChar,W		; W = SifRxCurChar - '0'
	addwf		RxCurArg,F		; RxCurArg += numeric value of SifRxCurChar
	incf		SifRxArgCharCount,F	; SifRxArgCharCount++
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

SIF_RX_STATE_PROC_ARG_GT3
	; Too big of a number, just abandon this command
	SET_SIF_RX_ST	SifRxWaitForTerm
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

SIF_RX_STATE_PROC_ARG_NOT_NUM
	; Look for a ' ' character
	movlw		' '
	subwf		SifRxCurChar,W
	btfss		STATUS,Z			; CurChar = ' '?
	goto		SIF_RX_STATE_PROC_ARG_NOT_SP	;    N: Check for term
	movf		SifRxArgCharCount,W		;    Y: Have we already seen a arg?
	btfss		STATUS,Z			; ArgCharCount = 0?
	goto		SIF_RX_STATE_PROC_ARG_SP_VLD	;    N: Got a valid argument
	pagesel		ISR_IF_RET			;    Y: Just intervening white space
	goto		ISR_IF_RET

SIF_RX_STATE_PROC_ARG_SP_VLD
	; Have a valid command and argument
	SET_SIF_RX_ST	SifRxValidCmd
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

SIF_RX_STATE_PROC_ARG_NOT_SP
	; Look for a term character
	movlw		CMD_TERM_CHAR
	subwf		SifRxCurChar,W
	btfss		STATUS,Z			; CurChar = TERM_CHAR?
	goto		SIF_RX_STATE_PROC_ARG_NOT_TRM	;    N: Unexpected character
	movf		SifRxArgCharCount,W		;    Y: Try to push this command
	btfsc		STATUS,Z			; ArgCharCount = 0?
	goto		SIF_RX_STATE_PROC_ARG_TRM_DONE	;    Y: No argument, drop this cmd
	call		PushRxCmd			;    N: Try to push this command

SIF_RX_STATE_PROC_ARG_TRM_DONE
	SET_SIF_RX_ST	SifRxIdle
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

SIF_RX_STATE_PROC_ARG_NOT_TRM
	; Unexpected character, abandon processing this command
	SET_SIF_RX_ST	SifRxWaitForTerm
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

SIF_RX_STATE_VALID_CMD
	; Got Valid Command State:
	;
	;   if (SifRxCurChar == ' ')
	;     do nothing
	;   else if (SifRxCurChar == TERM_CHAR)
	;     if (RxHasArg)
	;        try to push RxCurCmd and RxCurArg into CmdFifo
	;     else 
	;        try to push RxCurCmd into CmdFifo
	;     SifRxState = SifRxIdle
	;   else
	;     SifRxState = SifRxWaitForTerm
	;
	movlw		' '
	subwf		SifRxCurChar,W
	btfss		STATUS,Z			; CurChar = ' '?
	goto		SIF_RX_STATE_VALID_CMD_NOT_SP	;    N: Check for term
	pagesel		ISR_IF_RET			;    Y: Do nothing
	goto		ISR_IF_RET

SIF_RX_STATE_VALID_CMD_NOT_SP
	; Look for a term character
	movlw		CMD_TERM_CHAR
	subwf		SifRxCurChar,W
	btfss		STATUS,Z			; CurChar = TERM_CHAR?
	goto		SIF_RX_STATE_VALID_CMD_NOT_TRM	;    N: Unexpected character
	call		PushRxCmd			;    Y: Try to push this command
	SET_SIF_RX_ST	SifRxIdle
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

SIF_RX_STATE_VALID_CMD_NOT_TRM
	; Unexpected character, abandon processing this command
	SET_SIF_RX_ST	SifRxWaitForTerm
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

SIF_RX_STATE_WAIT_TERM
	; Wait for Termination State:
	;
	;   if (SifRxCurChar == TERM_CHAR)
	;     SifRxState = SifRxIdle
	;
	movlw		CMD_TERM_CHAR
	subwf		SifRxCurChar,W
	btfsc		STATUS,Z			; CurChar = TERM_CHAR?
	goto		SIF_RX_STATE_WAIT_TERM_TRM	;    Y: Saw term character
	pagesel		ISR_IF_RET			;    N: Do nothing
	goto		ISR_IF_RET

SIF_RX_STATE_WAIT_TERM_TRM
	; Saw term character
	SET_SIF_RX_ST	SifRxIdle
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET


; HANDLE_I2C_INT: I2C interrupt handler.  Implements a state-machine based
; on Microchip's AN734 "Using the PICmicro SSP for Slave I2C Communication".
; It has been modified to also check for entry with a Stop bit having been
; detected as well as a Start bit. It is called for any event on the I2C
; interface and uses the SSPSTAT to determine the correct state.
;
; On entry: SSPSTAT bits S, P, D_A, R_W and BF indicate state
;           I2cState indicates current logical state
;
; On exit: Appropriate action based on state including loading temperary
;            variables or the CmdFifo or transmitting a byte from the RspFifo
;          I2cState may be updated
;          Flags.RspFifoBusy and RspFifoPtr may be cleared
;          I2cFlags.I2cReadExpected and I2cFlags.I2cReadWaiting may be set
;
HANDLE_I2C_INT
	; Get the state-related bits for comparison checking
	bsf		STATUS,RP0
	movf		SSPSTAT,W
	andlw		B'00111101'	; Mask out unimportant bits in SSPSTAT
	bcf		STATUS,RP0
	movwf		I2cSspBits

I2C_INT_CH_STATE1
	; Check for State 1: Write operation, last byte was an address byte
	movlw		B'00001001'		; D_A=0, S=1, R_W=0, BF=1
	xorwf		I2cSspBits,W
	btfss		STATUS,Z		; Are we in state 1 with start bit?
	goto		I2C_INT_CH_STATE2	;   N: Check for state 2
						;   Y: State 1
I2C_INT_STATE1
	; Do a dummy read of the address in SSPBUF to clear BF
	movf		SSPBUF,W
	SET_I2C_ST	I2cRxCmd		; Note we expect command byte next
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

I2C_INT_CH_STATE2
	; Check for State 2: Write operation, last byte was a data byte
	; Note: We check for both start and stop bit conditions to allow the case
	;       where the master has ended the write before we get the SSPSTAT value
	movlw		B'00101001'		; D_A=1, S=1, R_W=0, BF=1
	xorwf		I2cSspBits,W
	btfsc		STATUS,Z		; Are we in state 2 with start bit?
	goto		I2C_INT_STATE2
	movlw		B'00110001'		; D_A=1, P=1, R_W=0, BF=1
	xorwf		I2cSspBits,W
	btfsc		STATUS,Z		; Are we in state 2 with stop bit?
	goto		I2C_INT_STATE2
	; --------------------- Begin Hack -------------------------
	; We unexpectedly see both Start and Stop flags set occasionally
	; so this code detects that case and hopes the contents of SSPBUF are valid
	;
	movlw		B'00111001'		; D_A=1, P=1, S=1, R_W=0, BF=1
	xorwf		I2cSspBits,W
	btfss		STATUS,Z		; Are we in state 2 with start and stop bit?
	goto		I2C_INT_CH_STATE3	;   N: Check for state 3
						;   Y: State 2
	; ---------------------  End Hack  -------------------------
I2C_INT_STATE2
	; Get the data byte and store it if appropriate
	btfsc		I2cState,I2cRxCmd	; Looking for Command?
	goto		I2C_INT_STATE2_CMD	;   Y: Check command
	btfsc		I2cState,I2cRxArg	;   N: Looking for Argument?
	goto		I2C_INT_STATE2_ARG	;      Y: Get Argument
						;      N: Skip subsequent data bytes
I2C_INT_STATE2_SKIP
	movf		SSPBUF,W		; Throw away data
	SET_I2C_ST	I2cIdle
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

I2C_INT_STATE2_CMD
	; Get the command byte and make sure it is valid
	movf		SSPBUF,W
	movwf		IsrTempData
	call		CheckValidCommand
	movwf		IsrTempData
	bcf		I2cFlags,I2cReadExpected	; Will be set if necessary
	btfsc		IsrTempData,.2			; Expect a response?
	bsf		I2cFlags,I2cReadExpected	;   Y: Store this for the read
	btfss		IsrTempData,.0			; Valid command?
	goto		I2C_INT_STATE2_SKIP		;   N: Done with this write sequence
	movf		SSPBUF,W			;   Y: Set RxCurCmd
	movwf		RxCurCmd
	bcf		SifRxState,RxHasArg		; ...In case we push this argument
	btfss		IsrTempData,.1			; Command has argument?
	goto		I2C_INT_STATE2_PUSH		;   N: Push command
	SET_I2C_ST	I2cRxArg			;   Y: Setup to get numeric argument
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

I2C_INT_STATE2_ARG
	; Get the argument byte and push the command
	movf		SSPBUF,W
	movwf		RxCurArg
	bsf		SifRxState,RxHasArg

I2C_INT_STATE2_PUSH
	; The I2C interface only allows one command at a time, we clear the RspFifo
	; in case there was any unread data from a previous command
	bcf		Flags,RspFifoBusy
	clrf		RspFifoPtr
	clrf		RspFifoCount
	bcf		I2cFlags,I2cReadWaiting

	; Push the command and optional argument
	call		PushRxCmd
	SET_I2C_ST	I2cIdle
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

I2C_INT_CH_STATE3
	; Check for State 3: Read operation, last byte was an address byte
	movlw		B'00001100'		; D_A=0, S=1, R_W=1, BF=0
	xorwf		I2cSspBits,W
	btfss		STATUS,Z		; Are we in state 3 with a start bit?
	goto		I2C_INT_CH_STATE4	;   N: Check for state 4
						;   Y: State 3
I2C_INT_STATE3
	; Determine if we have a data byte to transmit (we're now holding the I2C clock)
	btfss		I2cFlags,I2cReadExpected ; Should we have data?
	goto		I2C_INT_NO_RSP_DATA	;   N: Provide default 0 value
	btfss		Flags,RspFifoBusy	;   Y: Do we have it now?
	goto		I2C_INT_SET_DATA_WAIT	;       N: Setup so the command processing
						;          routine can provide the first byte
						;       Y: Try to load a byte of the data
	; Is there data to load?
	movf		RspFifoCount,W
	btfsc		STATUS,Z		; Have we processed all data?
	goto		I2C_INT_NO_RSP_DATA	;   Y: Provide default 0 value
						;   N: Get the next value
	bsf		STATUS,IRP		; RspFifo in upper banks
	movlw		RspFifo
	addwf		RspFifoPtr,W
	movwf		FSR
	incf		RspFifoPtr,F		; Increment to the next entry
	decf		RspFifoCount,F		;   and decrement the remaining count
	; Load the data into the SSP
	movf		INDF,W
	bcf		STATUS,IRP
	call		WriteI2c
	SET_I2C_ST	I2cTxData
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

I2C_INT_NO_RSP_DATA
	; We send 0 if there is no data in the RspFifo to send
	clrw
	call		WriteI2c
	SET_I2C_ST	I2cIdle
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

I2C_INT_SET_DATA_WAIT
	; We have the read request from the host but no data to give out so we
	; just let our SSP module hold the clock until the command processing routine
	; can provide the first byte when the RspFifo is full
	bsf		I2cFlags,I2cReadWaiting
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET

I2C_INT_CH_STATE4
	; Check for State 4: Read operation, last byte was a data byte
	movlw		B'00101100'		; D_A=1, S=1, R_W=1, BF=0
	xorwf		I2cSspBits,W
	btfsc		STATUS,Z		; Are we in state 4 with start bit?
	goto		I2C_INT_STATE3		;   Y: State 4 (same functionality as state 3)
						;   N: State 5 or other

I2C_INT_STATE5
	; This is either State 5 "A NACK was received when transmitting data back
	; from the master" or an illegal state, in either case we do nothing but
	; allow the SSP to automatcially reset itself and reset our state to idle.
	bsf		SSPCON,CKP		; Make sure SSP is running
	movf		SSPBUF,W		;   and buffer is empty
	SET_I2C_ST	I2cIdle
	pagesel		ISR_IF_RET
	goto		ISR_IF_RET


; WriteI2c: Load the I2C transmit buffer
;
; On entry: W contains the data to load
;
; On exit: Data has been loaded
WriteI2c
	bsf		STATUS,RP0
	btfsc		SSPSTAT,BF	; Is the buffer full?
	goto		WriteI2c	;    Y: Wait
	bcf		STATUS,RP0
DO_WRITE_I2C
	bcf		SSPCON,WCOL	; Clear the WCOL flag
	movwf		SSPBUF		; Load the byte from W
	btfsc		SSPCON,WCOL	; Was there a write collision?
	goto		DO_WRITE_I2C
	bsf		SSPCON,CKP	; Release the clock
	return


; CheckValidCommand: Check for a valid command.
;
; On entry: IsrTempData contains the command byte to check
;
; On exit: W contains the result
;            W[0] set for a valid command
;            W[1] set if the valid command has a subsequent argument
;            W[2] set if the command generates a response
;
; Note: Only called while in the ISR
;       Ordered so commands that require a subsequent argument are listed
;         first to reduce latency through ISR prior to argument
;
CheckValidCommand
	movlw		CMD_READ_ADC
	subwf		IsrTempData,W
	btfsc		STATUS,Z
	retlw		CMD_READ_ADC_ARG

	movlw		CMD_SET_IO_DIR
	subwf		IsrTempData,W
	btfsc		STATUS,Z
	retlw		CMD_SET_IO_DIR_ARG

	movlw		CMD_GET_HR
	subwf		IsrTempData,W
	btfsc		STATUS,Z
	retlw		CMD_GET_HR_ARG

	movlw		CMD_SET_IO_PINS
	subwf		IsrTempData,W
	btfsc		STATUS,Z
	retlw		CMD_SET_IO_PINS_ARG

	movlw		CMD_SET_MODE
	subwf		IsrTempData,W
	btfsc		STATUS,Z
	retlw		CMD_SET_MODE_ARG

	movlw		CMD_GET_IO_PINS
	subwf		IsrTempData,W
	btfsc		STATUS,Z
	retlw		CMD_GET_IO_PINS_ARG

	movlw		CMD_GET_MODE
	subwf		IsrTempData,W
	btfsc		STATUS,Z
	retlw		CMD_GET_MODE_ARG

	movlw		CMD_GET_VERSION
	subwf		IsrTempData,W
	btfsc		STATUS,Z
	retlw		CMD_GET_VERSION_ARG

	; Not a valid command
	retlw		0x00


; MultCurArgByTen: Multiply RxCurArg by 10 in preparation for adding in the
; next digit in an incoming ASCII numeric string.
;
; On entry: RxCurArg contains current value
;
; On exit: RxCurArg contains current value * 10
;
; Note: Only called while in the ISR
; 
MultCurArgByTen
	movf		RxCurArg,W
	addwf		RxCurArg,F		; x2
	addwf		RxCurArg,F		; x3
	addwf		RxCurArg,F		; x4
	addwf		RxCurArg,F		; x5
	bcf		STATUS,CARRY
	rlf		RxCurArg,F		; x10
	return


; PushRxCmd: Push a command byte and optionally a numeric argument byte
; into the CmdFifo if there is room.  Only complete commands are pushed.
; Commands are dropped if there is not room.
;
; On entry: RxCurCmd contains the validated command byte
;           RxCurArg contains the optional numeric argument
;           SifRxState.RxHasArg set if the RxCurArg variable is valid
;           CmdPushPtr points to the current CmdFifo location to load
;           CmdFifoCount contains the number of entries currently in the CmdFifo
;
; On exit: RxCurCmd and optionally RxCurArg loaded if there is room
;          CmdPushPtr updated
;          CmdFifoCount updated
;
; Note: Only called while in the ISR
;
PushRxCmd
	; !RxHasArg => push if (CmdFifoCount < CMD_FIFO_DEPTH)
	;  RxHasArg => push if (CmdFifoCount < (CMD_FIFO_DEPTH-1))
	clrw					
	btfsc		SifRxState,RxHasArg
	movlw		.1			; W = 1 if RxHasArg
	sublw		CMD_FIFO_DEPTH		; W = CMD_FIFO_DEPTH - {0 or 1}
	subwf		CmdFifoCount,W		; W = CmdFifoCount - (CMD_FIFO_DEPTH - {0 or 1})
	btfsc		STATUS,CARRY		; C = 0 if CmdFifoCount < ...
	return					;    C = 1: Can't push
						;    C = 0: Can push
	; Push the command byte
	movlw		CmdFifo
	addwf		CmdPushPtr,W
	movwf		FSR			; FSR = &CmdFifo + CmdPushPtr
	movf		RxCurCmd,W
	movwf		INDF			; Load Command byte
	incf		CmdPushPtr,W		; CmdPushPtr++
	andlw		CMD_FIFO_MASK
	movwf		CmdPushPtr
	incf		CmdFifoCount,F		; CmdFifoCount++

	; Push the argument byte if it exists
	btfss		SifRxState,RxHasArg
	return
	movlw		CmdFifo
	addwf		CmdPushPtr,W
	movwf		FSR			; FSR = &CmdFifo + CmdPushPtr
	movf		RxCurArg,W
	movwf		INDF			; Load Argument byte
	incf		CmdPushPtr,W		; CmdPushPtr++
	andlw		CMD_FIFO_MASK
	movwf		CmdPushPtr
	incf		CmdFifoCount,F		; CmdFifoCount++
	return


; HandleCommand: Process a complete command (with optional argument) from the
; CmdFifo.  Only process commands that require a response if the RspFifo is
; available (empty).  Control jumps to a specific command handler and then
; returns here.
;
; On entry: CmdFifo contains a complete command
;           CmdPopPtr points to the command
;           Flags.RspFifoBusy indicates if the RspFifo is empty or not
;
; On exit: Command may have been processed
;          RspFifo may have been loaded with a response
;          CmdPopPtr may have been incremented to the location past the command
;            and any argument
;
HandleCommand
	; Get the command
	movlw		CmdFifo
	addwf		CmdPopPtr,W
	movwf		FSR
	movf		INDF,W
	movwf		TempData		; TempData contains command
HANDLE_CMD_DECODE
	; Determine the action to take based on the command value. 
	;
	movlw		CMD_READ_ADC
	subwf		TempData,W
	btfsc		STATUS,Z
	goto		PROCESS_READ_ADC

	movlw		CMD_SET_IO_DIR
	subwf		TempData,W
	btfsc		STATUS,Z
	goto		PROCESS_SET_IO_DIR

	movlw		CMD_GET_HR
	subwf		TempData,W
	btfsc		STATUS,Z
	goto		PROCESS_GET_HR

	movlw		CMD_GET_IO_PINS
	subwf		TempData,W
	btfsc		STATUS,Z
	goto		PROCESS_GET_IO_PINS

	movlw		CMD_GET_MODE
	subwf		TempData,W
	btfsc		STATUS,Z
	goto		PROCESS_GET_MODE

	movlw		CMD_SET_IO_PINS
	subwf		TempData,W
	btfsc		STATUS,Z
	goto		PROCESS_SET_IO_PINS

	movlw		CMD_SET_MODE
	subwf		TempData,W
	btfsc		STATUS,Z
	goto		PROCESS_SET_MODE

	movlw		CMD_GET_VERSION
	subwf		TempData,W
	btfsc		STATUS,Z
	goto		PROCESS_GET_VERSION

HANDLE_CMD_BAD
	; Skip unknown commands
HANDLE_CMD_RET
	; Return point for commands that successfully executed
	call		IncCmdPopPtr
	return

HANDLE_CMD_SKIPPED_RET
	; Return point for commands which will not execute this evaluation
	return


; Handler to read the specified ADC channel
;
PROCESS_READ_ADC
	; Make sure the RspFifo is available
	btfsc		Flags,RspFifoBusy
	goto		HANDLE_CMD_SKIPPED_RET

	; Get the argument indicating which port to read
	call		IncCmdPopPtr		; CmdPopPtr points to argument
	movlw		CmdFifo
	addwf		CmdPopPtr,W
	movwf		FSR
	movf		INDF,W
	movwf		TempData

	; Make sure it is legal
	movlw		NUM_ADC_PORTS
	subwf		TempData,W		; W = TempData - NUM_ADC_PORTS
	btfsc		STATUS,CARRY		; C = 1 if TempData >= NUM_ADC_PORTS
	goto		HANDLE_CMD_RET		;    C = 1: Skip command
						;    C = 0: Process command

	; Setup the correct ADC port (replace ADCON0[4:2] with TempData[2:0])
	bcf		STATUS,CARRY
	rlf		TempData,F
	rlf		TempData,F		; TempData[4:2] contain channel
	movf		ADCON0,W
	andlw		0xE3
	iorwf		TempData,W
	movwf		ADCON0

	; Delay about 5 uSec to for Tacq
	;   At 8 MHz, each instruction is 500 nSec
	movlw		.3
	movwf		TempData
CHECK_ADC_TACQ_WAIT
	nop
	decfsz		TempData,F
	goto		CHECK_ADC_TACQ_WAIT

	; Trigger a reading and poll until the ADC is done
	bsf		ADCON0,ADC_GO
CHECK_ADC_LOOP
	btfsc		ADCON0,ADC_GO
	goto		CHECK_ADC_LOOP
	; Then get the result and push it into the RspFifo
	bsf		STATUS,IRP	; RspFifo in upper banks
	movlw		RspFifo
	movwf		FSR
	movf		ADRESH,W
	movwf		INDF		; Load ADC result 9:8
	incf		FSR,F
	bsf		STATUS,RP0
	movf		ADRESL,W
	movwf		INDF		; Load ADC result 7:0
	bcf		STATUS,RP0
	movlw		.2
	SET_RSP_FIFO_BUSY		; Set number of values in RspFifo
	bcf		STATUS,IRP

	goto		HANDLE_CMD_RET
	
	

; Handler to set the IO Pin directions
;
PROCESS_SET_IO_DIR
	; Get the argument containing the port mask
	call		IncCmdPopPtr		; CmdPopPtr points to argument
	movlw		CmdFifo
	addwf		CmdPopPtr,W
	movwf		FSR
	movf		INDF,W
	andlw		UTIL_PORT_MASK
	movwf		TempData

	; Modify the appropriate IO Pin directions
	bsf		STATUS,RP0
	movf		TRISC,W
	bcf		STATUS,RP0
	andlw		~UTIL_PORT_MASK
	iorwf		TempData,W
	bsf		STATUS,RP0
	movwf		TRISC
	bcf		STATUS,RP0

	goto		HANDLE_CMD_RET
	

; Handler to get Heartrate values
;
PROCESS_GET_HR
	; Make sure the RspFifo is available
	btfsc		Flags,RspFifoBusy
	goto		HANDLE_CMD_SKIPPED_RET

	; Get the argument indicating how many samples to return
	call		IncCmdPopPtr		; CmdPopPtr points to argument
	movlw		CmdFifo
	addwf		CmdPopPtr,W
	movwf		FSR
	movf		INDF,W
	movwf		PayloadCount

	; Get the requested number of heart rate values (limit to MAX_REQ_HR_VALS)
	incf		FSR,F
	movlw		MAX_REQ_HR_VALS+1
	subwf		PayloadCount,W	; W = NumHeartrateVals - MAX_REQ_HR_VALS+1
	movf		PayloadCount,W	; Reload W, will be overwritten if necessary
	btfsc		STATUS,CARRY	; C = 1 if NumHeartrateVals >= MAX_REQ_HR_VALS+1
	movlw		MAX_REQ_HR_VALS
	movwf		PayloadCount	; PayloadCount = NumHeartrateVals (0-MAX_REQ_HR_VALS)
	addwf		PayloadCount,F	; PayloadCount *= 2 (actual number of bytes)

	; Build the status field
	clrf		TempData
	btfss		Flags,MeasHr
	bsf		TempData,HR_FLAGS_NO_SIG

	; Load RspFifo with our response
	bsf		STATUS,IRP	; RspFifo and HrHistBuf in upper banks
	movlw		RspFifo
	movwf		FSR
	clrf		INDF
	incf		FSR,F
	movf		TempData,W
	movwf		INDF		; Byte 1:0: Status
	incf		FSR,F
	clrf		INDF
	incf		FSR,F
	movf		HeartbeatCount,W
	movwf		INDF		; Byte 3:2: Current timestamp or heartbeat count
	incf		FSR,F
	; Load the requested heart-rate values in the remaining bytes
	movf		PayloadCount,W		; Make sure PayloadCount != 0
	btfsc		STATUS,Z
	goto		PROCESS_GET_HR_DONE
	clrf		TempData		; Loop counter
PROCESS_GET_HR_LOOP
	; Get a HR value
	movlw		HrHistBuf
	addwf		TempData,W
	movwf		FSR			; FSR = current HR value location
	movf		INDF,W
	movwf		TempData2		; TempData2 = current HR value

	; Store it in the pkt buffer
	movlw		RspFifo
	addlw		.4
	addwf		TempData,W
	movwf		FSR			; FSR = current RspFifo location to load
	movf		TempData2,W
	movwf		INDF			; Store value
	; Process loop counter
	incf		TempData,F		; Increment to next location
	movf		TempData,W
	subwf		PayloadCount,W
	btfss		STATUS,Z		; TempData++ = PayloadCount?
	goto		PROCESS_GET_HR_LOOP	;    N: Do another

PROCESS_GET_HR_DONE
	movf		PayloadCount,W
	addlw		.4			; Add in Status and Count bytes
	SET_RSP_FIFO_BUSY			; Set number of values in RspFifo
	bcf		STATUS,IRP
	goto		HANDLE_CMD_RET


; Handler to get the IO Pin values
;
PROCESS_GET_IO_PINS
	; Make sure the RspFifo is available
	btfsc		Flags,RspFifoBusy
	goto		HANDLE_CMD_SKIPPED_RET

	; Then get the result and push it into the RspFifo
	bsf		STATUS,IRP	; RspFifo in upper banks
	movlw		RspFifo
	movwf		FSR
	clrf		INDF
	incf		FSR,F
	movf		PORT_C,W
	andlw		UTIL_PORT_MASK
	movwf		INDF		; Load IO Pins values
	movlw		.2
	SET_RSP_FIFO_BUSY		; Set number of values in RspFifo
	bcf		STATUS,IRP

	goto		HANDLE_CMD_RET


; Handler to get the operating Mode
;
PROCESS_GET_MODE
	; Make sure the RspFifo is available
	btfsc		Flags,RspFifoBusy
	goto		HANDLE_CMD_SKIPPED_RET

	; Then get the results and push them into the RspFifo
	bsf		STATUS,IRP	; RspFifo in upper banks
	movlw		RspFifo
	movwf		FSR
	clrf		INDF
	incf		FSR,F
	bsf		STATUS,RP0
	movf		TRISC,W
	andlw		UTIL_PORT_MASK
	movwf		INDF		; Load IO Pin direction values
	bcf		STATUS,RP0
	movlw		.2
	SET_RSP_FIFO_BUSY		; Set number of values in RspFifo
	bcf		STATUS,IRP

	goto		HANDLE_CMD_RET


; Handler to set the IO Pin values
;
PROCESS_SET_IO_PINS
	; Get the argument containing the port values
	call		IncCmdPopPtr		; CmdPopPtr points to argument
	movlw		CmdFifo
	addwf		CmdPopPtr,W
	movwf		FSR
	movf		INDF,W
	andlw		UTIL_PORT_MASK
	movwf		TempData

	; Modify the IO Pin values
	movf		PORT_C,W
	andlw		~UTIL_PORT_MASK
	iorwf		TempData,W
	movwf		PORT_C

	goto		HANDLE_CMD_RET
	

; Handler to set the specified Mode
;
PROCESS_SET_MODE
	; Get the argument containing the mode
	call		IncCmdPopPtr		; CmdPopPtr points to argument
	movlw		CmdFifo
	addwf		CmdPopPtr,W
	movwf		FSR
	movf		INDF,W
	andlw		0x07
	movwf		TempData

	; Store the Utility pin direction if requested
	btfss		TempData,.2
	goto		PROCESS_SET_MODE_DONE
	movlw		EEP_IO_DIR_ADDR
	movwf		FlashAddr
	bsf		STATUS,RP0
	movf		TRISC,W
	bcf		STATUS,RP0
	andlw		UTIL_PORT_MASK
	movwf		FlashData
	pagesel		WriteFlash
	call		WriteFlash
	; Store the Utility pin values
	movlw		EEP_IO_VAL_ADDR
	movwf		FlashAddr
	movf		PORT_C,W
	andlw		UTIL_PORT_MASK
	movwf		FlashData
	call		WriteFlash
	pagesel		$

PROCESS_SET_MODE_DONE
	goto		HANDLE_CMD_RET


; Handler to get the firmware type and version information
;
PROCESS_GET_VERSION
	; Make sure the RspFifo is available
	btfsc		Flags,RspFifoBusy
	goto		HANDLE_CMD_SKIPPED_RET

	; Then push the version information into the RspFifo
	bsf		STATUS,IRP	; RspFifo in upper banks
	movlw		RspFifo
	movwf		FSR
	clrf		INDF
	incf		FSR,F
	movlw		HRMI_DEV_TYPE
	movwf		INDF		; Load the firmware type
	incf		FSR,F
	clrf		INDF
	incf		FSR,F
	movlw		HRMI_VERSION
	movwf		INDF		; Load the firmware version
	movlw		.4
	SET_RSP_FIFO_BUSY		; Set number of values in RspFifo
	bcf		STATUS,IRP

	goto		HANDLE_CMD_RET


; IncCmdPopPtr: Increment the CmdPopPtr to the next location in CmdFifo
;
; On entry: CmdPopPtr points to current location
;           CmdFifoCount contains current number of valid entries
;
; On exit: CmdPopPtr points to next location
;          CmdFifoCount decremented
;
IncCmdPopPtr
	incf		CmdPopPtr,W		; CmdPopPtr++
	andlw		CMD_FIFO_MASK
	movwf		CmdPopPtr
	bcf		INTCON,GIE		; Disable interrupts
	movf		CmdFifoCount,W		; if CmdFifoCount != 0
	btfss		STATUS,Z		;   CmdFifoCount--
	decf		CmdFifoCount,F
	bsf		INTCON,GIE		; Re-enable interrupts
	return


; LoadFirstI2cReadData: Load the first byte from the RspFifo to satisfy
; a pending I2C read that has already been fielded by the I2C ISR routine.
; The I2C bus is being held waiting for the first data byte.
;
; On entry: RspFifoPtr points to the first entry
;           RspFifoCount contains the number of entries in the RspFifo
;           I2cFlags.I2cReadWaiting set
;
; On exit: SSPBUF loaded
;          SSPCON.CKP set to release the clock
;
LoadFirstI2cReadData
	; Just to be safe make sure there isn't an overflow condition
	movf		SSPBUF,W
	bcf		SSPCON,SSPOV
	; Load SSPBUF with the first byte of data
	bsf		STATUS,IRP		; RspFifo in upper banks
	movlw		RspFifo
	addwf		RspFifoPtr,W
	movwf		FSR
	movf		INDF,W
	bcf		STATUS,IRP
DO_LOAD_I2C_DATA
	bcf		SSPCON,WCOL		; Clear the WCOL flag
	movwf		SSPBUF			; Load the byte from W
	btfsc		SSPCON,WCOL		; Was there a write collision?
	goto		DO_LOAD_I2C_DATA
	bsf		SSPCON,CKP		; Release the clock
	; Update the RspFifo
	incf		RspFifoPtr,F		; Increment to the next entry
	decf		RspFifoCount,F		;   and decrement the remaining count
	return


; ProcessSifTx: Process data from the RspFifo and load one character into the
; USART transmit buffer.
;
; On entry: SifTxState contains current state
;           D3 - D0 contain current entry being transmitted
;           RspFifoCount contains current number of valid entries in RspFifo
;           RspFifoPtr points to next entry in RspFifo to process
;
; On exit: SifTxState updated
;          D3 - D0 may be loaded
;          RspFifoCount may be decremented
;          RspFifoPtr may be incremented or set to 0
;          Flags.RspFifoBusy may be cleared
;
; Note: Must be called when the transmit buffer is empty and only when we are
;       in serial mode
;
ProcessSifTx
	; Check if we are currently processing a character
	btfss		SifTxState,SifTxIdle
	goto		PROCESS_SIF_TX_OUTPUT_CHAR
	; Determine if we have a data byte pair to transmit
	movf		RspFifoCount,W
	btfsc		STATUS,Z
	goto		PROCESS_SIF_TX_END
	; Convert the current entry into its BCD components and determine
	; how many of them we need to transmit (leading zero suppression)
	clrf		SifTxState		; We will set the appropriate output bits
	bsf		STATUS,IRP		; RspFifo in upper banks
	movlw		RspFifo
	addwf		RspFifoPtr,W
	movwf		FSR
	movlw		.2
	addwf		RspFifoPtr,F		; Increment to the next entry
	subwf		RspFifoCount,F		;   and decrement the remaining count
	movf		INDF,W
	movwf		TempData		; TempData = [15:8]
	incf		FSR,F
	movf		INDF,W
	movwf		TempData2		; TempData2 = [7:0]
	bcf		STATUS,IRP
	call		Bin2DecFast		; Convert to BCD
	movf		D3,W
	btfsc		STATUS,Z
	goto		PROCESS_SIF_TX_TEST_H
	; Hundreds not zero so we know to send all four BCD digits
	bsf		SifTxState,SifTxOutputM
	bsf		SifTxState,SifTxOutputH
	bsf		SifTxState,SifTxOutputT
	bsf		SifTxState,SifTxOutputO
	goto		PROCESS_SIF_TX_OUTPUT_CHAR

PROCESS_SIF_TX_TEST_H
	; Thousands is zero, now check if hundreds is zero too
	movf		D2,W
	btfsc		STATUS,Z
	goto		PROCESS_SIF_TX_TEST_T
	; Hundreds not zero so we know to send remaining three BCD digits
	bsf		SifTxState,SifTxOutputH
	bsf		SifTxState,SifTxOutputT
	bsf		SifTxState,SifTxOutputO
	goto		PROCESS_SIF_TX_OUTPUT_CHAR

PROCESS_SIF_TX_TEST_T
	; Hundreds is zero, now check if tens is zero too
	movf		D1,W
	btfsc		STATUS,Z
	goto		PROCESS_SIF_TX_O
	; Tens not zero so we know to send the lower two BCD digits
	bsf		SifTxState,SifTxOutputT
	bsf		SifTxState,SifTxOutputO
	goto		PROCESS_SIF_TX_OUTPUT_CHAR

PROCESS_SIF_TX_O
	; Tens is zero too so only send ones
	bsf		SifTxState,SifTxOutputO
	goto		PROCESS_SIF_TX_OUTPUT_CHAR

PROCESS_SIF_TX_END
	; We have transmitted all data bytes, now send the terminating <CR>
	; setup RspFifoPtr for next time and clear RspFifoBusy
	movlw		CMD_TERM_CHAR
	movwf		TXREG
	clrf		RspFifoPtr
	bcf		Flags,RspFifoBusy
	return

PROCESS_SIF_TX_OUTPUT_CHAR
	; Determine which digit to output
	btfss		SifTxState,SifTxOutputM
	goto		PROCESS_SIF_TX_CHECK_H
	; Output the thousands digit
	movf		D3,W
	addlw		'0'			; Convert to ASCII
	movwf		TXREG
	bcf		SifTxState,SifTxOutputM
	return

PROCESS_SIF_TX_CHECK_H
	btfss		SifTxState,SifTxOutputH
	goto		PROCESS_SIF_TX_CHECK_T
	; Output the hundreds digit
	movf		D2,W
	addlw		'0'			; Convert to ASCII
	movwf		TXREG
	bcf		SifTxState,SifTxOutputH
	return

PROCESS_SIF_TX_CHECK_T
	btfss		SifTxState,SifTxOutputT
	goto		PROCESS_SIF_TX_CHECK_O
	; Output the tens digit
	movf		D1,W
	addlw		'0'			; Convert to ASCII
	movwf		TXREG
	bcf		SifTxState,SifTxOutputT
	return

PROCESS_SIF_TX_CHECK_O
	btfss		SifTxState,SifTxOutputO
	goto		PROCESS_SIF_TX_SEND_SP
	; Output the ones digit
	movf		D0,W
	addlw		'0'			; Convert to ASCII
	movwf		TXREG
	bcf		SifTxState,SifTxOutputO
	return

PROCESS_SIF_TX_SEND_SP
	; We've sent all BCD digits so send a space in case there is another
	; data value following this one and set SifTxIdle to indicate we're done
	; with a entry
	movlw		' '
	movwf		TXREG
	bsf		SifTxState,SifTxIdle
	return


; Bin2DecFast: Convert a 12-bit binary number into a 4-digit BCD result.
; Based on a routine created by Peter Hemsley and found on the internet at
; http://www.piclist.com/techref/microchip/math/radix/b2bu-10b4d-eag.htm.
; 
; On entry: TempData:TempData2 contain the 12-bit value (TempData[7:4] assumed 0)
;
; On exit: D3 - D0 contains the BCD digits (msb - lsb)
;          SifTxTensOnes contain tens ([7:4]) and ones ([3:0] BCD digits
;
Bin2DecFast
        movf    TempData,w
        iorlw   0xF0            ;w=H2-16
        movwf   D1              ;D1=H2-16
        addwf   D1,f            ;D1=H2*2-32
        addwf   D1,f            ;D1=H2*3-48
        movwf   D2              ;D2=H2-16
        addlw   -D'5'           ;w=H2-21
        addwf   D2,f            ;D2=H2*2-37 Done!
        addlw   D'41'           ;w=H2+20
        movwf   D0              ;D0=H2+20

        swapf   TempData2,w
        iorlw   0xF0            ;w=H1-16
        addwf   D1,f            ;D1=H2*3+H1-64
        addwf   D0,f            ;D0=H2+H1+4, C=1
        rlf     D0,f            ;D0=(H2+H1)*2+9, C=0
        comf    D0,f            ;D0=-(H2+H1)*2-10
        rlf     D0,f            ;D0=-(H2+H1)*4-20

        movf    TempData2,w
        andlw   0x0F            ;w=H0
        addwf   D0,f            ;D0=H0-(H2+H1)*4-20 Done!
        rlf     D1,f            ;C=0, D1=H2*6+H1*2-128 Done! 

        movlw   D'5'
        movwf   D3

        movlw   D'10'
mod0
        addwf   D0,f            ;D(X)=D(X)mod10
        decf    D1,f            ;D(X+1)=D(X+1)+D(X)div10
        skpc
        goto    mod0
mod1
        addwf   D1,f
        decf    D2,f
        skpc
        goto    mod1
mod2
        addwf   D2,f
        decf    D3,f
        skpc
        goto    mod2

        return


	end
